# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

# pretrained infos
pretrained_minigpt4_infos = {
    # llama
    "default-minigpt4": {
        "blip2_config_path": "https://huggingface.co/Salesforce/blip2-flan-t5-xxl/resolve/main/config.json",
        "llama_config_path": "https://huggingface.co/wangrongsheng/MiniGPT-4-LLaMA-7B/resolve/main/config.json",
        "vocab": "https://huggingface.co/wangrongsheng/MiniGPT-4-LLaMA-7B/resolve/main/tokenizer.model",
        "vision_config": "https://huggingface.co/Salesforce/blip2-flan-t5-xxl/resolve/main/preprocessor_config.json",
    },
    "minigpt4-7b": {
        "blip2_config_path": "https://huggingface.co/Salesforce/blip2-flan-t5-xxl/resolve/main/config.json",
        "llama_config_path": "https://huggingface.co/wangrongsheng/MiniGPT-4-LLaMA-7B/resolve/main/config.json",
        "vocab": "https://huggingface.co/wangrongsheng/MiniGPT-4-LLaMA-7B/resolve/main/tokenizer.model",
        "vision_config": "https://huggingface.co/Salesforce/blip2-flan-t5-xxl/resolve/main/preprocessor_config.json",
        "weight": [
            "https://huggingface.co/Salesforce/blip2-flan-t5-xxl/resolve/main/pytorch_model-00001-of-00006.bin",
            "https://huggingface.co/fuliucansheng/minigpt4/resolve/main/pytorch_model.vicuna7b.bin",
            "https://huggingface.co/wangrongsheng/MiniGPT-4-LLaMA-7B/resolve/main/pytorch_model-00001-of-00002.bin",
            "https://huggingface.co/wangrongsheng/MiniGPT-4-LLaMA-7B/resolve/main/pytorch_model-00002-of-00002.bin",
        ],
    },
    "minigpt4-13b": {
        "blip2_config_path": "https://huggingface.co/Salesforce/blip2-flan-t5-xxl/resolve/main/config.json",
        "llama_config_path": "https://huggingface.co/wangrongsheng/MiniGPT-4-LLaMA/resolve/main/config.json",
        "vocab": "https://huggingface.co/wangrongsheng/MiniGPT-4-LLaMA/resolve/main/tokenizer.model",
        "vision_config": "https://huggingface.co/Salesforce/blip2-flan-t5-xxl/resolve/main/preprocessor_config.json",
        "weight": [
            "https://huggingface.co/Salesforce/blip2-flan-t5-xxl/resolve/main/pytorch_model-00001-of-00006.bin",
            "https://huggingface.co/fuliucansheng/minigpt4/resolve/main/pytorch_model.vicuna13b.bin",
            "https://huggingface.co/wangrongsheng/MiniGPT-4-LLaMA/resolve/main/pytorch_model-00001-of-00003.bin",
            "https://huggingface.co/wangrongsheng/MiniGPT-4-LLaMA/resolve/main/pytorch_model-00002-of-00003.bin",
            "https://huggingface.co/wangrongsheng/MiniGPT-4-LLaMA/resolve/main/pytorch_model-00003-of-00003.bin",
        ],
    },
}

import unitorch.cli.models.minigpt4.modeling
import unitorch.cli.models.minigpt4.processing
from unitorch.cli.models.minigpt4.modeling import MiniGPT4Blip2LlamaForGeneration
from unitorch.cli.models.minigpt4.processing import MiniGPT4Blip2LlamaProcessor



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import sys
import logging
import inspect

OPTIMIZED_CLASSES = dict()


# replace decorator from fastseq
def replace(target_obj):
    """A decorator to replace the specified obj.

    `target_obj` can be a class or a function.

    Example:

    ```python
    class A:
        def f(self):
            print('class A')
    @replace(A)
    class B:
        def f(self):
            print('class B')
    ```

    Args:
        target_obj (class/func/method): a class, method, or function to be
                                        replaced.

    Returns:
        A decorator function to replace the input object.
    """

    def decorator(new_obj):
        if target_obj in OPTIMIZED_CLASSES:
            logging.warning("{} has been optimized again.".format(target_obj))
        setattr(new_obj, "__replaced_class__", target_obj)
        OPTIMIZED_CLASSES[target_obj] = new_obj
        for k, v in list(sys.modules.items()):
            if (
                target_obj.__name__ in v.__dict__
                and v.__dict__[target_obj.__name__] is target_obj
            ):
                delattr(sys.modules[k], target_obj.__name__)
                setattr(sys.modules[k], target_obj.__name__, new_obj)
                logging.debug(
                    "In module {}, {} is replaced by {}".format(k, target_obj, new_obj)
                )
            # replace target_obj if it is used as the base classes.
            for key in list(v.__dict__.keys()):
                if (
                    inspect.isclass(v.__dict__[key])
                    and v.__dict__[key] != new_obj
                    and target_obj in v.__dict__[key].__bases__
                ):
                    idx = v.__dict__[key].__bases__.index(target_obj)
                    bases = list(v.__dict__[key].__bases__)
                    bases[idx] = new_obj
                    v.__dict__[key].__bases__ = tuple(bases)
                    logging.debug(
                        "In module {}, the base class of {} is replaced by {}".format(
                            k, v.__dict__[key], new_obj
                        )
                    )
        return new_obj

    return decorator



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import json
import torch
import torch.nn.functional as F
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
import diffusers.schedulers as schedulers
import diffusers.pipelines.stable_video_diffusion as stable_video_diffusion
from transformers import (
    CLIPTextConfig,
    CLIPTextModel,
    CLIPVisionConfig,
    CLIPVisionModelWithProjection,
)
from diffusers.schedulers import SchedulerMixin
from diffusers.models.attention_processor import LoRAAttnProcessor, LoRAAttnProcessor2_0
from diffusers.models import (
    UNet2DModel,
    UNet2DConditionModel,
    AutoencoderKL,
    AutoencoderKLTemporalDecoder,
    UNetSpatioTemporalConditionModel,
)
from diffusers.pipelines import (
    StableDiffusionPipeline,
    StableDiffusionImg2ImgPipeline,
    StableDiffusionInpaintPipeline,
    StableDiffusionUpscalePipeline,
    StableDiffusionDepth2ImgPipeline,
    StableVideoDiffusionPipeline,
)
from unitorch.models import (
    GenericModel,
    GenericOutputs,
    QuantizationConfig,
    QuantizationMixin,
)

stable_video_diffusion.pipeline_stable_video_diffusion.tensor2vid = (
    lambda video, *args, **kwargs: video
)


def compute_snr(timesteps, noise_scheduler):
    """
    Computes SNR as per https://github.com/TiankaiHang/Min-SNR-Diffusion-Training/blob/521b624bd70c67cee4bdf49225915f5945a872e3/guided_diffusion/gaussian_diffusion.py#L847-L849
    """
    alphas_cumprod = noise_scheduler.alphas_cumprod
    sqrt_alphas_cumprod = alphas_cumprod**0.5
    sqrt_one_minus_alphas_cumprod = (1.0 - alphas_cumprod) ** 0.5
    # Expand the tensors.
    # Adapted from https://github.com/TiankaiHang/Min-SNR-Diffusion-Training/blob/521b624bd70c67cee4bdf49225915f5945a872e3/guided_diffusion/gaussian_diffusion.py#L1026
    sqrt_alphas_cumprod = sqrt_alphas_cumprod.to(device=timesteps.device)[
        timesteps
    ].float()
    while len(sqrt_alphas_cumprod.shape) < len(timesteps.shape):
        sqrt_alphas_cumprod = sqrt_alphas_cumprod[..., None]
    alpha = sqrt_alphas_cumprod.expand(timesteps.shape)

    sqrt_one_minus_alphas_cumprod = sqrt_one_minus_alphas_cumprod.to(
        device=timesteps.device
    )[timesteps].float()
    while len(sqrt_one_minus_alphas_cumprod.shape) < len(timesteps.shape):
        sqrt_one_minus_alphas_cumprod = sqrt_one_minus_alphas_cumprod[..., None]
    sigma = sqrt_one_minus_alphas_cumprod.expand(timesteps.shape)

    # Compute SNR
    snr = (alpha / sigma) ** 2
    return snr


class GenericStableModel(GenericModel, QuantizationMixin):
    prefix_keys_in_state_dict = {
        # unet weights
        "^conv_in.*": "unet.",
        "^conv_norm_out.*": "unet.",
        "^conv_out.*": "unet.",
        "^time_embedding.*": "unet.",
        "^class_embedding.*": "unet.",
        "^up_blocks.*": "unet.",
        "^mid_block.*": "unet.",
        "^down_blocks.*": "unet.",
        # text weights
        "^text_model.*": "text.",
        # vae weights
        "^encoder.*": "vae.",
        "^decoder.*": "vae.",
        "^post_quant_conv.*": "vae.",
        "^quant_conv.*": "vae.",
    }

    replace_keys_in_state_dict = {
        "\.query\.": ".to_q.",
        "\.key\.": ".to_k.",
        "\.value\.": ".to_v.",
        "\.proj_attn\.": ".to_out.0.",
    }

    def __init__(
        self,
        config_path: str,
        text_config_path: str,
        vae_config_path: str,
        scheduler_config_path: str,
        quant_config_path: Optional[str] = None,
        image_size: Optional[int] = None,
        in_channels: Optional[int] = None,
        out_channels: Optional[int] = None,
        num_train_timesteps: Optional[int] = 1000,
        num_infer_timesteps: Optional[int] = 50,
        freeze_vae_encoder: Optional[bool] = True,
        freeze_text_encoder: Optional[bool] = True,
        snr_gamma: Optional[float] = 5.0,
        lora_r: Optional[int] = None,
        seed: Optional[int] = 1123,
    ):
        super().__init__()
        self.seed = seed
        self.num_train_timesteps = num_train_timesteps
        self.num_infer_timesteps = num_infer_timesteps
        self.image_size = image_size
        self.snr_gamma = snr_gamma

        config_dict = json.load(open(config_path))
        if image_size is not None:
            config_dict.update({"sample_size": image_size})
        if in_channels is not None:
            config_dict.update({"in_channels": in_channels})
        if out_channels is not None:
            config_dict.update({"out_channels": out_channels})
        self.unet = UNet2DConditionModel.from_config(config_dict)

        text_config = CLIPTextConfig.from_json_file(text_config_path)
        self.text = CLIPTextModel(text_config)

        vae_config_dict = json.load(open(vae_config_path))
        self.vae = AutoencoderKL.from_config(vae_config_dict)

        scheduler_config_dict = json.load(open(scheduler_config_path))
        scheduler_class_name = scheduler_config_dict.get("_class_name", "DDPMScheduler")
        assert hasattr(schedulers, scheduler_class_name)
        scheduler_class = getattr(schedulers, scheduler_class_name)
        assert issubclass(scheduler_class, SchedulerMixin)
        scheduler_config_dict["num_train_timesteps"] = num_train_timesteps
        self.scheduler = scheduler_class.from_config(scheduler_config_dict)

        self.vae_scale_factor = 2 ** (len(self.vae.config.block_out_channels) - 1)

        if freeze_vae_encoder:
            for param in self.vae.parameters():
                param.requires_grad = False

        if freeze_text_encoder:
            for param in self.text.parameters():
                param.requires_grad = False

        if quant_config_path is not None:
            self.quant_config = QuantizationConfig.from_json_file(quant_config_path)
            self.quantize(self.quant_config, ignore_modules=["lm_head", "unet", "vae"])

        if lora_r is not None:
            for param in self.unet.parameters():
                param.requires_grad = False
            self.enable_lora(lora_r=lora_r)

        self.scheduler.set_timesteps(num_inference_steps=self.num_infer_timesteps)

    def enable_lora(self, lora_r: Optional[int] = 4):
        lora_attn_procs = {}
        for name in self.unet.attn_processors.keys():
            cross_attention_dim = (
                None
                if name.endswith("attn1.processor")
                else self.unet.config.cross_attention_dim
            )
            if name.startswith("mid_block"):
                hidden_size = self.unet.config.block_out_channels[-1]
            elif name.startswith("up_blocks"):
                block_id = int(name[len("up_blocks.")])
                hidden_size = list(reversed(self.unet.config.block_out_channels))[
                    block_id
                ]
            elif name.startswith("down_blocks"):
                block_id = int(name[len("down_blocks.")])
                hidden_size = self.unet.config.block_out_channels[block_id]

            lora_attn_procs[name] = LoRAAttnProcessor(
                hidden_size=hidden_size,
                cross_attention_dim=cross_attention_dim,
                rank=lora_r,
            )

        self.unet.set_attn_processor(lora_attn_procs)


class StableForText2ImageGeneration(GenericStableModel):
    def __init__(
        self,
        config_path: str,
        text_config_path: str,
        vae_config_path: str,
        scheduler_config_path: str,
        quant_config_path: Optional[str] = None,
        image_size: Optional[int] = None,
        in_channels: Optional[int] = None,
        out_channels: Optional[int] = None,
        num_train_timesteps: Optional[int] = 1000,
        num_infer_timesteps: Optional[int] = 50,
        freeze_vae_encoder: Optional[bool] = True,
        freeze_text_encoder: Optional[bool] = True,
        snr_gamma: Optional[float] = 5.0,
        lora_r: Optional[int] = None,
        seed: Optional[int] = 1123,
    ):
        super().__init__(
            config_path=config_path,
            text_config_path=text_config_path,
            vae_config_path=vae_config_path,
            scheduler_config_path=scheduler_config_path,
            quant_config_path=quant_config_path,
            image_size=image_size,
            in_channels=in_channels,
            out_channels=out_channels,
            num_train_timesteps=num_train_timesteps,
            num_infer_timesteps=num_infer_timesteps,
            freeze_vae_encoder=freeze_vae_encoder,
            freeze_text_encoder=freeze_text_encoder,
            snr_gamma=snr_gamma,
            lora_r=lora_r,
            seed=seed,
        )

        self.pipeline = StableDiffusionPipeline(
            vae=self.vae,
            text_encoder=self.text,
            unet=self.unet,
            scheduler=self.scheduler,
            tokenizer=None,
            safety_checker=None,
            feature_extractor=None,
        )
        self.pipeline.set_progress_bar_config(disable=True)

    def forward(
        self,
        pixel_values: torch.Tensor,
        input_ids: torch.Tensor,
        attention_mask: Optional[torch.Tensor] = None,
    ):
        latents = self.vae.encode(pixel_values).latent_dist.sample()
        latents = latents * self.vae.config.scaling_factor
        noise = torch.randn(latents.shape).to(latents.device)
        batch = latents.size(0)

        timesteps = torch.randint(
            0,
            self.scheduler.config.num_train_timesteps,
            (batch,),
            device=pixel_values.device,
        ).long()

        noise_latents = self.scheduler.add_noise(
            latents,
            noise,
            timesteps,
        )

        encoder_hidden_states = self.text(input_ids)[0]
        # encoder_hidden_states = self.text(input_ids, attention_mask)[0]
        outputs = self.unet(
            noise_latents,
            timesteps,
            encoder_hidden_states,
        ).sample
        if self.scheduler.config.prediction_type == "v_prediction":
            noise = self.scheduler.get_velocity(latents, noise, timesteps)
        if self.snr_gamma > 0:
            snr = compute_snr(timesteps, self.scheduler)
            base_weight = (
                torch.stack(
                    [snr, self.snr_gamma * torch.ones_like(timesteps)], dim=1
                ).min(dim=1)[0]
                / snr
            )

            if self.scheduler.config.prediction_type == "v_prediction":
                mse_loss_weights = base_weight + 1
            else:
                mse_loss_weights = base_weight
            loss = F.mse_loss(outputs, noise, reduction="none")
            loss = loss.mean(dim=list(range(1, len(loss.shape)))) * mse_loss_weights
            loss = loss.mean()
        else:
            loss = F.mse_loss(outputs, noise, reduction="mean")
        return loss

    def generate(
        self,
        input_ids: torch.Tensor,
        negative_input_ids: torch.Tensor,
        attention_mask: Optional[torch.Tensor] = None,
        negative_attention_mask: Optional[torch.Tensor] = None,
        height: Optional[int] = 512,
        width: Optional[int] = 512,
        guidance_scale: Optional[float] = 7.5,
    ):
        prompt_embeds = self.text(
            input_ids,
            # attention_mask,
        )[0]
        negative_prompt_embeds = self.text(
            negative_input_ids,
            # negative_attention_mask,
        )[0]

        images = self.pipeline(
            prompt_embeds=prompt_embeds,
            negative_prompt_embeds=negative_prompt_embeds,
            generator=torch.Generator(device=self.pipeline.device).manual_seed(
                self.seed
            ),
            height=height,
            width=width,
            guidance_scale=guidance_scale,
            output_type="np.array",
        ).images

        return GenericOutputs(images=torch.from_numpy(images))


class StableForImage2ImageGeneration(GenericStableModel):
    def __init__(
        self,
        config_path: str,
        text_config_path: str,
        vae_config_path: str,
        scheduler_config_path: str,
        quant_config_path: Optional[str] = None,
        image_size: Optional[int] = None,
        in_channels: Optional[int] = None,
        out_channels: Optional[int] = None,
        num_train_timesteps: Optional[int] = 1000,
        num_infer_timesteps: Optional[int] = 50,
        freeze_vae_encoder: Optional[bool] = True,
        freeze_text_encoder: Optional[bool] = True,
        snr_gamma: Optional[float] = 5.0,
        lora_r: Optional[int] = None,
        seed: Optional[int] = 1123,
    ):
        super().__init__(
            config_path=config_path,
            text_config_path=text_config_path,
            vae_config_path=vae_config_path,
            scheduler_config_path=scheduler_config_path,
            quant_config_path=quant_config_path,
            image_size=image_size,
            in_channels=in_channels,
            out_channels=out_channels,
            num_train_timesteps=num_train_timesteps,
            num_infer_timesteps=num_infer_timesteps,
            freeze_vae_encoder=freeze_vae_encoder,
            freeze_text_encoder=freeze_text_encoder,
            snr_gamma=snr_gamma,
            lora_r=lora_r,
            seed=seed,
        )

        self.pipeline = StableDiffusionImg2ImgPipeline(
            vae=self.vae,
            text_encoder=self.text,
            unet=self.unet,
            scheduler=self.scheduler,
            tokenizer=None,
            safety_checker=None,
            feature_extractor=None,
        )
        self.pipeline.set_progress_bar_config(disable=True)

    def forward(
        self,
    ):
        raise NotImplementedError

    def generate(
        self,
        input_ids: torch.Tensor,
        negative_input_ids: torch.Tensor,
        pixel_values: torch.Tensor,
        attention_mask: Optional[torch.Tensor] = None,
        negative_attention_mask: Optional[torch.Tensor] = None,
        strength: Optional[float] = 0.8,
        guidance_scale: Optional[float] = 7.5,
    ):
        prompt_embeds = self.text(
            input_ids,
            # attention_mask,
        )[0]
        negative_prompt_embeds = self.text(
            negative_input_ids,
            # negative_attention_mask,
        )[0]

        images = self.pipeline(
            image=pixel_values,
            prompt_embeds=prompt_embeds,
            negative_prompt_embeds=negative_prompt_embeds,
            generator=torch.Generator(device=self.pipeline.device).manual_seed(
                self.seed
            ),
            strength=strength,
            guidance_scale=guidance_scale,
            output_type="np.array",
        ).images

        return GenericOutputs(images=torch.from_numpy(images))


class StableForImageInpainting(GenericStableModel):
    def __init__(
        self,
        config_path: str,
        text_config_path: str,
        vae_config_path: str,
        scheduler_config_path: str,
        quant_config_path: Optional[str] = None,
        image_size: Optional[int] = None,
        in_channels: Optional[int] = None,
        out_channels: Optional[int] = None,
        num_train_timesteps: Optional[int] = 1000,
        num_infer_timesteps: Optional[int] = 50,
        freeze_vae_encoder: Optional[bool] = True,
        freeze_text_encoder: Optional[bool] = True,
        snr_gamma: Optional[float] = 5.0,
        lora_r: Optional[int] = None,
        seed: Optional[int] = 1123,
    ):
        super().__init__(
            config_path=config_path,
            text_config_path=text_config_path,
            vae_config_path=vae_config_path,
            scheduler_config_path=scheduler_config_path,
            quant_config_path=quant_config_path,
            image_size=image_size,
            in_channels=in_channels,
            out_channels=out_channels,
            num_train_timesteps=num_train_timesteps,
            num_infer_timesteps=num_infer_timesteps,
            freeze_vae_encoder=freeze_vae_encoder,
            freeze_text_encoder=freeze_text_encoder,
            snr_gamma=snr_gamma,
            lora_r=lora_r,
            seed=seed,
        )

        self.pipeline = StableDiffusionInpaintPipeline(
            vae=self.vae,
            text_encoder=self.text,
            unet=self.unet,
            scheduler=self.scheduler,
            tokenizer=None,
            safety_checker=None,
            feature_extractor=None,
        )
        self.pipeline.set_progress_bar_config(disable=True)

    def forward(
        self,
    ):
        raise NotImplementedError

    def generate(
        self,
        input_ids: torch.Tensor,
        negative_input_ids: torch.Tensor,
        pixel_values: torch.Tensor,
        pixel_masks: torch.Tensor,
        attention_mask: Optional[torch.Tensor] = None,
        negative_attention_mask: Optional[torch.Tensor] = None,
        strength: Optional[float] = 1.0,
        guidance_scale: Optional[float] = 7.5,
    ):
        prompt_embeds = self.text(
            input_ids,
            # attention_mask,
        )[0]
        negative_prompt_embeds = self.text(
            negative_input_ids,
            # negative_attention_mask,
        )[0]

        images = self.pipeline(
            image=pixel_values,
            mask_image=pixel_masks,
            prompt_embeds=prompt_embeds,
            negative_prompt_embeds=negative_prompt_embeds,
            generator=torch.Generator(device=self.pipeline.device).manual_seed(
                self.seed
            ),
            strength=strength,
            guidance_scale=guidance_scale,
            output_type="np.array",
        ).images

        return GenericOutputs(images=torch.from_numpy(images))


class StableForImageResolution(GenericStableModel):
    def __init__(
        self,
        config_path: str,
        text_config_path: str,
        vae_config_path: str,
        scheduler_config_path: str,
        quant_config_path: Optional[str] = None,
        image_size: Optional[int] = None,
        in_channels: Optional[int] = None,
        out_channels: Optional[int] = None,
        num_train_timesteps: Optional[int] = 1000,
        num_infer_timesteps: Optional[int] = 50,
        freeze_vae_encoder: Optional[bool] = True,
        freeze_text_encoder: Optional[bool] = True,
        snr_gamma: Optional[float] = 5.0,
        lora_r: Optional[int] = None,
        seed: Optional[int] = 1123,
    ):
        super().__init__(
            config_path=config_path,
            text_config_path=text_config_path,
            vae_config_path=vae_config_path,
            scheduler_config_path=scheduler_config_path,
            quant_config_path=quant_config_path,
            image_size=image_size,
            in_channels=in_channels,
            out_channels=out_channels,
            num_train_timesteps=num_train_timesteps,
            num_infer_timesteps=num_infer_timesteps,
            freeze_vae_encoder=freeze_vae_encoder,
            freeze_text_encoder=freeze_text_encoder,
            snr_gamma=snr_gamma,
            lora_r=lora_r,
            seed=seed,
        )

        self.pipeline = StableDiffusionUpscalePipeline(
            vae=self.vae,
            text_encoder=self.text,
            unet=self.unet,
            scheduler=self.scheduler,
            low_res_scheduler=self.scheduler,
            tokenizer=None,
            safety_checker=None,
            feature_extractor=None,
        )
        self.pipeline.set_progress_bar_config(disable=True)

    def forward(
        self,
    ):
        raise NotImplementedError

    def generate(
        self,
        input_ids: torch.Tensor,
        negative_input_ids: torch.Tensor,
        pixel_values: torch.Tensor,
        attention_mask: Optional[torch.Tensor] = None,
        negative_attention_mask: Optional[torch.Tensor] = None,
        guidance_scale: Optional[float] = 9.0,
        noise_level: Optional[int] = 20,
    ):
        prompt_embeds = self.text(
            input_ids,
            # attention_mask,
        )[0]
        negative_prompt_embeds = self.text(
            negative_input_ids,
            # negative_attention_mask,
        )[0]

        images = self.pipeline(
            image=pixel_values,
            prompt_embeds=prompt_embeds,
            negative_prompt_embeds=negative_prompt_embeds,
            generator=torch.Generator(device=self.pipeline.device).manual_seed(
                self.seed
            ),
            guidance_scale=guidance_scale,
            noise_level=noise_level,
            output_type="np.array",
        ).images

        return GenericOutputs(images=torch.from_numpy(images))


class StableForImage2VideoGeneration(GenericModel):
    def __init__(
        self,
        config_path: str,
        vision_config_path: str,
        vae_config_path: str,
        scheduler_config_path: str,
        quant_config_path: Optional[str] = None,
        image_size: Optional[int] = None,
        in_channels: Optional[int] = None,
        out_channels: Optional[int] = None,
        num_train_timesteps: Optional[int] = 1000,
        num_infer_timesteps: Optional[int] = 50,
        freeze_vae_encoder: Optional[bool] = True,
        freeze_text_encoder: Optional[bool] = True,
        snr_gamma: Optional[float] = 5.0,
        lora_r: Optional[int] = None,
        seed: Optional[int] = 1123,
    ):
        super().__init__()

        config_dict = json.load(open(config_path))
        if image_size is not None:
            config_dict.update({"sample_size": image_size})
        if in_channels is not None:
            config_dict.update({"in_channels": in_channels})
        if out_channels is not None:
            config_dict.update({"out_channels": out_channels})
        self.unet = UNetSpatioTemporalConditionModel.from_config(config_dict)

        vision_config = CLIPVisionConfig.from_json_file(vision_config_path)
        self.vision = CLIPVisionModelWithProjection(vision_config)

        vae_config_dict = json.load(open(vae_config_path))
        self.vae = AutoencoderKLTemporalDecoder.from_config(vae_config_dict)

        scheduler_config_dict = json.load(open(scheduler_config_path))
        scheduler_class_name = scheduler_config_dict.get("_class_name", "DDPMScheduler")
        assert hasattr(schedulers, scheduler_class_name)
        scheduler_class = getattr(schedulers, scheduler_class_name)
        assert issubclass(scheduler_class, SchedulerMixin)
        scheduler_config_dict["num_train_timesteps"] = num_train_timesteps
        self.scheduler = scheduler_class.from_config(scheduler_config_dict)

        self.vae_scale_factor = 2 ** (len(self.vae.config.block_out_channels) - 1)

        self.pipeline = StableVideoDiffusionPipeline(
            vae=self.vae,
            image_encoder=self.vision,
            unet=self.unet,
            scheduler=self.scheduler,
            low_res_scheduler=self.scheduler,
            tokenizer=None,
            safety_checker=None,
            feature_extractor=None,
        )
        self.pipeline.set_progress_bar_config(disable=True)

    def forward(
        self,
    ):
        raise NotImplementedError

    def generate(
        self,
        pixel_values: torch.Tensor,
    ):
        frames = self.pipeline(
            image=pixel_values,
            output_type="pt",
        ).frames

        return GenericOutputs(frames=frames)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from hmac import new
import os
import torch
import numpy as np
from PIL import Image
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from torchvision.transforms import Resize, CenterCrop, ToTensor, Normalize, Compose
from transformers import DetrFeatureExtractor
from unitorch.utils import resize_shortest_edge, pop_value
from unitorch.models import GenericOutputs


class DetrProcessor(object):
    def __init__(
        self,
        vision_config_path: str,
        min_size_test: Optional[int] = 800,
        max_size_test: Optional[int] = 1333,
    ):
        """
        Args:
            vision_config_path: vision config path to detr processor
            min_size_test: resize shortest edge parameters
            max_size_test: resize shortest edge parameters
        """
        self.vision_processor = DetrFeatureExtractor.from_json_file(vision_config_path)

        self.image_mean = self.vision_processor.image_mean
        self.image_std = self.vision_processor.image_std
        self.min_size_test = min_size_test
        self.max_size_test = max_size_test

    def image(
        self,
        image: Image.Image,
    ):
        """
        Args:
            image: input image
        """
        width, height = image.size
        image = resize_shortest_edge(
            image.convert("RGB"),
            [self.min_size_test, self.max_size_test],
            self.max_size_test,
        )

        image = np.array(image) / 255.0

        image = self.vision_processor.normalize(
            image=image,
            mean=self.image_mean,
            std=self.image_std,
        )
        return GenericOutputs(
            image=torch.tensor(image).permute(2, 0, 1),
            sizes=torch.tensor([height, width]),
        )

    def detection(
        self,
        image: Image.Image,
        bboxes: List[List[float]],
        classes: List[int],
    ):
        """
        Args:
            image: input image
            bboxes: bboxes to image
            classes: class to each bbox
        """
        outputs = self.image(image)
        # new_h, new_w = outputs.image.size()[1:]
        org_h, org_w = outputs.sizes
        image = outputs.image
        bboxes = torch.tensor(bboxes).float()
        bboxes[:, 0] = bboxes[:, 0] / org_w
        bboxes[:, 1] = bboxes[:, 1] / org_h
        bboxes[:, 2] = bboxes[:, 2] / org_w
        bboxes[:, 3] = bboxes[:, 3] / org_h
        classes = torch.tensor(classes)
        if bboxes.dim() == 1:
            bboxes = bboxes.unsqueeze(0)

        assert (
            bboxes.size(-1) == 4 and classes.dim() == 1 and len(classes) == len(bboxes)
        )
        return GenericOutputs(image=image, bboxes=bboxes, classes=classes)

    def segmentation(
        self,
        image: Image.Image,
        gt_image: Image.Image,
        num_classes: Optional[int] = None,
    ):
        """
        Args:
            image: input image
            gt_image: ground truth image
            num_classes: num classes to classification
        """
        image = self.image(image).image
        gt_image = np.array(gt_image)
        if num_classes is not None:
            gt_image = np.minimum(gt_image, num_classes)
        return GenericOutputs(
            image=torch.tensor(image),
            gt_image=torch.tensor(gt_image),
        )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

pretrained_peft_infos = {}

import unitorch.cli.models.peft.modeling_bloom
import unitorch.cli.models.peft.modeling_llama
import unitorch.cli.models.peft.modeling_mistral
from unitorch.cli.models.peft.modeling_bloom import (
    BloomLoraForClassification,
    BloomLoraForGeneration,
)
from unitorch.cli.models.peft.modeling_llama import (
    LlamaLoraForClassification,
    LlamaLoraForGeneration,
)
from unitorch.cli.models.peft.modeling_mistral import MistralLoraForGeneration



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import random
import logging
from functools import partial
from PIL import Image
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union

import numpy as np
import torch

from transformers import LlamaTokenizer
from transformers import BlipImageProcessor

from unitorch.utils import pop_value, truncate_sequence_pair
from unitorch.models import (
    HfTextClassificationProcessor,
    HfTextGenerationProcessor,
    HfImageClassificationProcessor,
    GenericOutputs,
)


class MiniGPT4Blip2LlamaProcessor(
    HfTextClassificationProcessor,
    HfImageClassificationProcessor,
    HfTextGenerationProcessor,
):
    """
    MiniGPT4Blip2LlamaProcessor is a class for processing inputs and outputs of the MiniGPT4 model with Blip2 and Llama.
    It inherits from the _MiniGPT4Blip2LlamaProcessor class.
    """

    def __init__(
        self,
        vocab_file: str,
        vision_config_path: str,
        max_prefix_seq_length: Optional[int] = 64,
        max_suffix_seq_length: Optional[int] = 128,
        max_gen_seq_length: Optional[int] = 48,
    ):
        """
        Initializes a MiniGPT4Blip2LlamaProcessor instance.

        Args:
            vocab_path (str): The file path to the vocabulary.
            vision_config_path (str): The file path to the vision configuration.
            max_prefix_seq_length (int, optional): The maximum length of the prefix sequence. Defaults to 64.
            max_suffix_seq_length (int, optional): The maximum length of the suffix sequence. Defaults to 128.
            max_gen_seq_length (int, optional): The maximum length of the generated sequence. Defaults to 48.
        """
        tokenizer = LlamaTokenizer(vocab_file=vocab_file)
        tokenizer.cls_token = tokenizer.bos_token
        tokenizer.sep_token = tokenizer.eos_token
        tokenizer.pad_token = tokenizer.unk_token
        tokenizer.cls_token_id = tokenizer.bos_token_id
        tokenizer.sep_token_id = tokenizer.eos_token_id
        tokenizer.pad_token_id = tokenizer.unk_token_id
        vision_processor = BlipImageProcessor.from_json_file(vision_config_path)
        HfTextClassificationProcessor.__init__(
            self,
            tokenizer=tokenizer,
            max_seq_length=max_prefix_seq_length,
        )
        HfTextGenerationProcessor.__init__(
            self,
            tokenizer=tokenizer,
            max_seq_length=max_prefix_seq_length,
            max_gen_seq_length=max_gen_seq_length,
        )
        HfImageClassificationProcessor.__init__(self, vision_processor=vision_processor)
        self.max_prefix_seq_length = max_prefix_seq_length
        self.max_suffix_seq_length = max_suffix_seq_length

    def prompt(
        self,
        prefix_text: str,
        suffix_text: str,
        image: Union[Image.Image, str],
        max_prefix_seq_length: Optional[int] = None,
        max_suffix_seq_length: Optional[int] = None,
    ):
        """
        Process text as a prompt.

        Args:
            prefix_text (str): The prefix text.
            suffix_text (str): The suffix text.
            image (PIL.Image.Image): The input image.
            max_prefix_seq_length (int, optional): The maximum length of the prefix sequence. Defaults to None.
            max_suffix_seq_length (int, optional): The maximum length of the suffix sequence. Defaults to None.

        Returns:
            GenericOutputs: Processed input_ids tensor.
        """
        max_prefix_seq_length = pop_value(
            max_prefix_seq_length,
            self.max_prefix_seq_length,
        )
        max_suffix_seq_length = pop_value(
            max_suffix_seq_length,
            self.max_suffix_seq_length,
        )
        prefix_tokens = self.tokenizer.tokenize(str(prefix_text))

        if len(prefix_tokens) >= max_prefix_seq_length:
            logging.warning(
                f"Input prefix text {prefix_text} has been truncated to {max_prefix_seq_length - 1} tokens."
            )
            prefix_tokens = prefix_tokens[: max_prefix_seq_length - 1]

        suffix_tokens = self.tokenizer.tokenize(str(suffix_text))

        if len(suffix_tokens) > max_suffix_seq_length:
            logging.warning(
                f"Input suffix text {suffix_text} has been truncated to {max_suffix_seq_length} tokens."
            )
            suffix_tokens = suffix_tokens[:max_suffix_seq_length]

        prefix_tokens = [self.bos_token] + prefix_tokens
        prefix_padding = [self.pad_token] * (max_prefix_seq_length - len(prefix_tokens))
        prefix_tokens = prefix_padding + prefix_tokens
        prefix_input_ids = self.tokenizer.convert_tokens_to_ids(prefix_tokens)

        suffix_padding = [self.pad_token] * (max_suffix_seq_length - len(suffix_tokens))
        suffix_tokens = suffix_padding + suffix_tokens
        suffix_input_ids = self.tokenizer.convert_tokens_to_ids(suffix_tokens)

        outputs = HfImageClassificationProcessor.classification(
            self,
            image=image,
        )

        return GenericOutputs(
            prefix_input_ids=torch.tensor(prefix_input_ids, dtype=torch.long),
            suffix_input_ids=torch.tensor(suffix_input_ids, dtype=torch.long),
            pixel_values=outputs.pixel_values,
        )

    def generation_inputs(
        self,
        prefix_text: str,
        suffix_text: str,
        image: Union[Image.Image, str],
        max_prefix_seq_length: Optional[int] = None,
        max_suffix_seq_length: Optional[int] = None,
    ):
        """
        Process text for generation inputs.

        Args:
            prefix_text (str): The prefix text.
            suffix_text (str): The suffix text.
            image (PIL.Image.Image): The input image.
            max_prefix_seq_length (int, optional): The maximum length of the prefix sequence. Defaults to None.
            max_suffix_seq_length (int, optional): The maximum length of the suffix sequence. Defaults to None.

        Returns:
            GenericOutputs: Processed input_ids tensor.
        """
        max_prefix_seq_length = pop_value(
            max_prefix_seq_length,
            self.max_prefix_seq_length,
        )
        max_suffix_seq_length = pop_value(
            max_suffix_seq_length,
            self.max_suffix_seq_length,
        )
        prefix_tokens = self.tokenizer.tokenize(str(prefix_text))

        if len(prefix_tokens) >= max_prefix_seq_length:
            logging.warning(
                f"Input prefix text {prefix_text} has been truncated to {max_prefix_seq_length - 1} tokens."
            )
            prefix_tokens = prefix_tokens[: max_prefix_seq_length - 1]

        suffix_tokens = self.tokenizer.tokenize(str(suffix_text))

        if len(suffix_tokens) > max_suffix_seq_length:
            logging.warning(
                f"Input suffix text {suffix_text} has been truncated to {max_suffix_seq_length} tokens."
            )
            suffix_tokens = suffix_tokens[:max_suffix_seq_length]

        prefix_tokens = [self.bos_token] + prefix_tokens
        prefix_padding = [self.pad_token] * (max_prefix_seq_length - len(prefix_tokens))
        prefix_tokens = prefix_padding + prefix_tokens
        prefix_input_ids = self.tokenizer.convert_tokens_to_ids(prefix_tokens)

        suffix_padding = [self.pad_token] * (max_suffix_seq_length - len(suffix_tokens))
        suffix_tokens = suffix_padding + suffix_tokens
        suffix_input_ids = self.tokenizer.convert_tokens_to_ids(suffix_tokens)

        outputs = HfImageClassificationProcessor.classification(
            self,
            image=image,
        )

        return GenericOutputs(
            prefix_input_ids=torch.tensor(prefix_input_ids, dtype=torch.long),
            suffix_input_ids=torch.tensor(suffix_input_ids, dtype=torch.long),
            pixel_values=outputs.pixel_values,
        )

    def generation_labels(
        self,
        text: str,
        max_gen_seq_length: Optional[int] = None,
    ):
        """
        Process text for generation labels.

        Args:
            text (str): Input text.
            max_gen_seq_length (int, optional): Maximum generation sequence length. Defaults to None.

        Returns:
            GenericOutputs: Processed input_ids and attention_mask tensors.
        """
        max_gen_seq_length = pop_value(
            max_gen_seq_length,
            self.max_gen_seq_length,
        )
        tokens = self.tokenizer.tokenize(str(text))[: max_gen_seq_length - 1] + [
            self.eos_token
        ]
        padding = [self.pad_token] * (max_gen_seq_length - len(tokens))
        input_ids = self.tokenizer.convert_tokens_to_ids(tokens)
        attention_mask = [1] * len(input_ids)

        padding = [0] * (max_gen_seq_length - len(input_ids))
        input_ids += [self.pad_token_id] * len(padding)
        attention_mask += padding

        assert len(input_ids) == max_gen_seq_length
        assert len(attention_mask) == max_gen_seq_length
        return GenericOutputs(
            input_ids=torch.tensor(input_ids, dtype=torch.long),
            attention_mask=torch.tensor(attention_mask, dtype=torch.long),
        )

    def generation(
        self,
        prefix_text: str,
        suffix_text: str,
        text_pair: str,
        image: Union[Image.Image, str],
        max_prefix_seq_length: Optional[int] = None,
        max_suffix_seq_length: Optional[int] = None,
        max_gen_seq_length: Optional[int] = None,
    ):
        """
        Process text for generation.

        Args:
            prefix_text (str): The prefix text.
            suffix_text (str): The suffix text.
            text_pair (str): The text pair.
            image (PIL.Image.Image): The input image.
            max_prefix_seq_length (int, optional): The maximum length of the prefix sequence. Defaults to None.
            max_suffix_seq_length (int, optional): The maximum length of the suffix sequence. Defaults to None.
            max_gen_seq_length (int, optional): The maximum length of the generated sequence. Defaults to None.

        Returns:
            GenericOutputs: Processed input_ids, attention_mask, input_ids_label, and attention_mask_label tensors.
        """
        max_prefix_seq_length = pop_value(
            max_prefix_seq_length,
            self.max_prefix_seq_length,
        )
        max_suffix_seq_length = pop_value(
            max_suffix_seq_length,
            self.max_suffix_seq_length,
        )
        max_gen_seq_length = pop_value(
            max_gen_seq_length,
            self.max_gen_seq_length,
        )
        prefix_tokens = self.tokenizer.tokenize(str(prefix_text))

        if len(prefix_tokens) >= max_prefix_seq_length:
            logging.warning(
                f"Input prefix text {prefix_text} has been truncated to {max_prefix_seq_length - 1} tokens."
            )
            prefix_tokens = prefix_tokens[: max_prefix_seq_length - 1]

        suffix_tokens = self.tokenizer.tokenize(str(suffix_text))

        if len(suffix_tokens) > max_suffix_seq_length:
            logging.warning(
                f"Input suffix text {suffix_text} has been truncated to {max_suffix_seq_length} tokens."
            )
            suffix_tokens = suffix_tokens[:max_suffix_seq_length]

        prefix_tokens = [self.bos_token] + prefix_tokens
        prefix_padding = [self.pad_token] * (max_prefix_seq_length - len(prefix_tokens))
        prefix_attention_mask = [0] * len(prefix_padding) + [1] * len(prefix_tokens)
        prefix_tokens = prefix_padding + prefix_tokens
        prefix_input_ids = self.tokenizer.convert_tokens_to_ids(prefix_tokens)

        suffix_padding = [self.pad_token] * (max_suffix_seq_length - len(suffix_tokens))
        suffix_attention_mask = [0] * len(suffix_padding) + [1] * len(suffix_tokens)
        suffix_tokens = suffix_padding + suffix_tokens
        suffix_input_ids = self.tokenizer.convert_tokens_to_ids(suffix_tokens)

        tokens_pair = self.tokenizer.tokenize(str(text_pair))[
            : max_gen_seq_length - 1
        ] + [self.eos_token]

        padding_pair = [self.pad_token] * (max_gen_seq_length - len(tokens_pair))
        input_ids_pair = self.tokenizer.convert_tokens_to_ids(
            tokens_pair + padding_pair
        )
        attention_mask_pair = [1] * len(tokens_pair) + [0] * len(padding_pair)

        tokens_label = tokens_pair + [self.pad_token] * (
            max_gen_seq_length - len(tokens_pair) + 1
        )
        input_ids_label = self.tokenizer.convert_tokens_to_ids(tokens_label)
        input_ids_label = [0] * (max_suffix_seq_length - 1) + input_ids_label
        attention_mask_label = [1] * len(tokens_pair) + [0] * (
            max_gen_seq_length - len(tokens_pair) + 1
        )
        attention_mask_label = [0] * (max_suffix_seq_length - 1) + attention_mask_label

        outputs = HfImageClassificationProcessor.classification(
            self,
            image=image,
        )

        return GenericOutputs(
            prefix_input_ids=torch.tensor(prefix_input_ids, dtype=torch.long),
            prefix_attention_mask=torch.tensor(prefix_attention_mask, dtype=torch.long),
            suffix_input_ids=torch.tensor(suffix_input_ids, dtype=torch.long),
            suffix_attention_mask=torch.tensor(suffix_attention_mask, dtype=torch.long),
            input_ids_pair=torch.tensor(input_ids_pair, dtype=torch.long),
            attention_mask_pair=torch.tensor(attention_mask_pair, dtype=torch.long),
            input_ids_label=torch.tensor(input_ids_label, dtype=torch.long),
            attention_mask_label=torch.tensor(attention_mask_label, dtype=torch.long),
            pixel_values=outputs.pixel_values,
        )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from unitorch.models.bloom.modeling import (
    BloomForClassification,
    BloomForPretrain,
    BloomForGeneration,
)
from unitorch.models.bloom.processing import BloomProcessor



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import random
from functools import partial
from random import randint, shuffle, choice

import numpy as np
import torch
from transformers import BertTokenizer

from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from unitorch.utils import pop_value
from unitorch.models.bert import BertProcessor, get_bert_tokenizer


class VisualBertProcessor(BertProcessor):
    """
    Processor for VisualBERT-based models.
    """

    def __init__(
        self,
        vocab_path,
        max_seq_length: Optional[int] = 128,
        special_input_ids: Optional[Dict] = dict(),
        do_lower_case: Optional[bool] = True,
        do_basic_tokenize: Optional[bool] = True,
        do_whole_word_mask: Optional[bool] = True,
        masked_lm_prob: Optional[float] = 0.15,
        max_predictions_per_seq: Optional[int] = 20,
    ):
        """
        Initializes the VisualBertProcessor.

        Args:
            vocab_path (str): Path to the vocabulary file.
            max_seq_length (Optional[int]): Maximum sequence length. Defaults to 128.
            special_input_ids (Optional[Dict]): Special input IDs. Defaults to an empty dictionary.
            do_lower_case (Optional[bool]): Whether to convert text to lowercase. Defaults to True.
            do_basic_tokenize (Optional[bool]): Whether to perform basic tokenization. Defaults to True.
            do_whole_word_mask (Optional[bool]): Whether to use whole word masking. Defaults to True.
            masked_lm_prob (Optional[float]): Probability for masked LM. Defaults to 0.15.
            max_predictions_per_seq (Optional[int]): Maximum number of masked LM predictions per sequence. Defaults to 20.
        """
        super().__init__(
            vocab_path=vocab_path,
            max_seq_length=max_seq_length,
            do_lower_case=do_lower_case,
            do_basic_tokenize=do_basic_tokenize,
            do_whole_word_mask=do_whole_word_mask,
            masked_lm_prob=masked_lm_prob,
            max_predictions_per_seq=max_predictions_per_seq,
        )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from PIL import Image
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from unitorch.utils import pop_value, nested_dict_value
from unitorch.models.diffusers import Blip2Processor as _Blip2Processor
from unitorch.cli import (
    cached_path,
    add_default_section_for_init,
    add_default_section_for_function,
    register_process,
)
from unitorch.cli.models import (
    TensorsInputs,
)
from unitorch.cli.models.diffusers import pretrained_diffusers_infos


class Blip2Processor(_Blip2Processor):
    def __init__(
        self,
        vocab_path: str,
        merge_path: str,
        vision_config_path: str,
        vae_config_path: str,
        max_seq_length: Optional[int] = 77,
        position_start_id: Optional[int] = 0,
        pad_token: Optional[str] = "<|endoftext|>",
        image_size: Optional[int] = 512,
        center_crop: Optional[bool] = False,
    ):
        super().__init__(
            vocab_path=vocab_path,
            merge_path=merge_path,
            vision_config_path=vision_config_path,
            vae_config_path=vae_config_path,
            max_seq_length=max_seq_length,
            position_start_id=position_start_id,
            pad_token=pad_token,
            image_size=image_size,
            center_crop=center_crop,
        )

    @classmethod
    @add_default_section_for_init("core/process/diffusers/blip2")
    def from_core_configure(cls, config, **kwargs):
        config.set_default_section("core/process/diffusers/blip2")
        pretrained_name = config.getoption(
            "pretrained_name", "stable-v1.5-blipdiffuion"
        )
        pretrain_infos = nested_dict_value(pretrained_diffusers_infos, pretrained_name)

        vocab_path = config.getoption("vocab_path", None)
        vocab_path = pop_value(
            vocab_path,
            nested_dict_value(pretrain_infos, "text", "vocab"),
        )
        vocab_path = cached_path(vocab_path)

        merge_path = config.getoption("merge_path", None)
        merge_path = pop_value(
            merge_path,
            nested_dict_value(pretrain_infos, "text", "merge"),
        )
        merge_path = cached_path(merge_path)

        vision_config_path = config.getoption("vision_config_path", None)
        vision_config_path = pop_value(
            vision_config_path,
            nested_dict_value(pretrain_infos, "vision_config"),
        )
        vision_config_path = cached_path(vision_config_path)

        vae_config_path = config.getoption("vae_config_path", None)
        vae_config_path = pop_value(
            vae_config_path,
            nested_dict_value(pretrain_infos, "vae", "config"),
        )
        vae_config_path = cached_path(vae_config_path)

        return {
            "vocab_path": vocab_path,
            "merge_path": merge_path,
            "vision_config_path": vision_config_path,
            "vae_config_path": vae_config_path,
        }

    @register_process("core/process/diffusers/blip2/text2image")
    def _text2image(
        self,
    ):
        raise NotImplementedError

    @register_process("core/process/diffusers/blip2/text2image/inputs")
    def _text2image_inputs(
        self,
        prompt: str,
        refer_prompt: str,
        refer_image: Union[Image.Image, str],
        negative_prompt: Optional[str] = "",
        max_seq_length: Optional[int] = None,
    ):
        outputs = super().text2image_inputs(
            prompt=prompt,
            refer_prompt=refer_prompt,
            refer_image=refer_image,
            negative_prompt=negative_prompt,
            max_seq_length=max_seq_length,
        )
        return TensorsInputs(
            input_ids=outputs.input_ids,
            attention_mask=outputs.attention_mask,
            negative_input_ids=outputs.negative_input_ids,
            negative_attention_mask=outputs.negative_attention_mask,
            refer_input_ids=outputs.refer_input_ids,
            refer_attention_mask=outputs.refer_attention_mask,
            refer_pixel_values=outputs.refer_pixel_values,
        )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

# pretrained infos
pretrained_beit_infos = {
    "default-beit": {
        "config": "https://huggingface.co/microsoft/beit-base-patch16-224-pt22k-ft22k/resolve/main/config.json",
        "vision_config": "https://huggingface.co/microsoft/beit-base-patch16-224-pt22k-ft22k/resolve/main/preprocessor_config.json",
    },
    "beit-base-patch16-224-pt22k-ft22k": {
        "config": "https://huggingface.co/microsoft/beit-base-patch16-224-pt22k-ft22k/resolve/main/config.json",
        "vision_config": "https://huggingface.co/microsoft/beit-base-patch16-224-pt22k-ft22k/resolve/main/preprocessor_config.json",
        "weight": "https://huggingface.co/microsoft/beit-base-patch16-224-pt22k-ft22k/resolve/main/pytorch_model.bin",
    },
    "beit-base-patch16-224": {
        "config": "https://huggingface.co/microsoft/beit-base-patch16-224/resolve/main/config.json",
        "vision_config": "https://huggingface.co/microsoft/beit-base-patch16-224/resolve/main/preprocessor_config.json",
        "weight": "https://huggingface.co/microsoft/beit-base-patch16-224/resolve/main/pytorch_model.bin",
    },
}

import unitorch.cli.models.beit.modeling
import unitorch.cli.models.beit.processing
from unitorch.cli.models.beit.modeling import BeitForImageClassification
from unitorch.cli.models.beit.processing import BeitProcessor



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import torch
import logging
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from unitorch.optims.lion import Lion
from unitorch.models import CheckpointMixin
from unitorch.cli import add_default_section_for_init, register_optim


@register_optim("core/optim/lion")
class LionOptimizer(Lion, CheckpointMixin):
    def __init__(
        self,
        params,
        learning_rate: Optional[float] = 0.00001,
    ):
        super().__init__(
            params=params,
            lr=learning_rate,
        )

    @classmethod
    @add_default_section_for_init("core/optim/lion")
    def from_core_configure(cls, config, **kwargs):
        pass



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import sys
import fire
import logging
import importlib
import uvicorn
import unitorch.cli
from fastapi import FastAPI
from transformers.utils import is_remote_url
from unitorch.cli import CoreConfigureParser
from unitorch.cli import (
    import_library,
    cached_path,
    set_global_config,
    registered_fastapi,
    init_registered_module,
)
import unitorch.cli.fastapis


@fire.decorators.SetParseFn(str)
def fastapi(config_path_or_dir: str, **kwargs):
    config_file = kwargs.pop("config_file", "config.ini")

    if os.path.isdir(config_path_or_dir):
        config_path = os.path.join(config_path_or_dir, config_file)
        sys.path.insert(0, config_path_or_dir)
        for f in os.listdir(config_path_or_dir):
            fpath = os.path.normpath(os.path.join(config_path_or_dir, f))
            if (
                not f.startswith("_")
                and not f.startswith(".")
                and (f.endswith(".py") or os.path.isdir(fpath))
            ):
                fname = f[:-3] if f.endswith(".py") else f
                module = importlib.import_module(f"{fname}")
    else:
        config_path = cached_path(config_path_or_dir)

    params = []
    for k, v in kwargs.items():
        if k.count("@") > 0:
            k0 = k.split("@")[0]
            k1 = "@".join(k.split("@")[1:])
        else:
            k0 = "core/cli"
            k1 = k
        params.append((k0, k1, v))

    if config_path is not None:
        config = CoreConfigureParser(config_path, params=params)
    else:
        config = CoreConfigureParser(params=params)

    set_global_config(config)

    depends_libraries = config.getdefault("core/cli", "depends_libraries", None)

    if depends_libraries:
        for library in depends_libraries:
            import_library(library)

    enabled_services = config.getdefault("core/cli", "enabled_services", None)
    assert enabled_services is not None
    if isinstance(enabled_services, str):
        enabled_services = [enabled_services]
    assert all(
        fastapi_service in registered_fastapi for fastapi_service in enabled_services
    )
    fastapi_instances = [
        registered_fastapi.get(fastapi_service)["obj"](config)
        for fastapi_service in enabled_services
    ]
    app = FastAPI()

    for fastapi_instance in fastapi_instances:
        fastapi_instance.start()
        app.include_router(fastapi_instance.router)

    uvicorn.run(app, host="0.0.0.0", port=5000, log_level="info")


def cli_main():
    fire.Fire(fastapi)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import torch
import numpy as np
from PIL import Image
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from torchvision.transforms import Resize, CenterCrop, ToTensor, Normalize, Compose
from transformers import BlipImageProcessor
from transformers.image_utils import to_numpy_array, ChannelDimension
from transformers.image_transforms import to_channel_dimension_format
from unitorch.utils import pop_value
from unitorch.models import (
    HfTextClassificationProcessor,
    HfTextGenerationProcessor,
    HfImageClassificationProcessor,
    GenericOutputs,
)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

# pretrained infos
pretrained_detr_infos = {
    "default-detr": {
        "config": "https://huggingface.co/facebook/detr-resnet-50/resolve/main/config.json",
        "vision_config": "https://huggingface.co/facebook/detr-resnet-50/resolve/main/preprocessor_config.json",
    },
    "detr-resnet-50": {
        "config": "https://huggingface.co/facebook/detr-resnet-50/resolve/main/config.json",
        "vision_config": "https://huggingface.co/facebook/detr-resnet-50/resolve/main/preprocessor_config.json",
        "weight": "https://huggingface.co/facebook/detr-resnet-50/resolve/main/pytorch_model.bin",
    },
}

import unitorch.cli.models.detr.modeling
import unitorch.cli.models.detr.processing
from unitorch.cli.models.detr.modeling import (
    DetrForDetection,
)
from unitorch.cli.models.detr.processing import DetrProcessor



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import torch
import collections
import math
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from unitorch.scores import convert_tensor_to_strings, remove_strings_ignore_tokens


def _get_ngrams(segment, max_order):
    """Extracts all n-grams upto a given maximum order from an input segment.
    Args:
        segment: text segment from which n-grams will be extracted.
        max_order: maximum length in tokens of the n-grams returned by this
            methods.
    Returns:
        The Counter containing all n-grams upto max_order in segment
        with a count of how many times each n-gram occurred.
    """
    ngram_counts = collections.Counter()
    for order in range(1, max_order + 1):
        for i in range(0, len(segment) - order + 1):
            ngram = tuple(segment[i : i + order])
            ngram_counts[ngram] += 1
    return ngram_counts


def _compute_bleu(reference_corpus, translation_corpus, max_order=4, smooth=False):
    """Computes BLEU score of translated segments against one or more references.
    Args:
        reference_corpus: list of lists of references for each translation. Each
            reference should be tokenized into a list of tokens.
        translation_corpus: list of translations to score. Each translation
            should be tokenized into a list of tokens.
        max_order: Maximum n-gram order to use when computing BLEU score.
        smooth: Whether or not to apply Lin et al. 2004 smoothing.
    Returns:
        3-Tuple with the BLEU score, n-gram precisions, geometric mean of n-gram
        precisions and brevity penalty.
    """
    matches_by_order = [0] * max_order
    possible_matches_by_order = [0] * max_order
    reference_length = 0
    translation_length = 0
    for references, translation in zip(reference_corpus, translation_corpus):
        reference_length += min(len(r) for r in references)
        translation_length += len(translation)

        merged_ref_ngram_counts = collections.Counter()
        for reference in references:
            merged_ref_ngram_counts |= _get_ngrams(reference, max_order)
            translation_ngram_counts = _get_ngrams(translation, max_order)
            overlap = translation_ngram_counts & merged_ref_ngram_counts
        for ngram in overlap:
            matches_by_order[len(ngram) - 1] += overlap[ngram]
        for order in range(1, max_order + 1):
            possible_matches = len(translation) - order + 1
            if possible_matches > 0:
                possible_matches_by_order[order - 1] += possible_matches

    precisions = [0] * max_order
    for i in range(0, max_order):
        if smooth:
            precisions[i] = (matches_by_order[i] + 1.0) / (
                possible_matches_by_order[i] + 1.0
            )
        else:
            if possible_matches_by_order[i] > 0:
                precisions[i] = (
                    float(matches_by_order[i]) / possible_matches_by_order[i]
                )
            else:
                precisions[i] = 0.0

    if min(precisions) > 0:
        p_log_sum = sum((1.0 / max_order) * math.log(p) for p in precisions)
        geo_mean = math.exp(p_log_sum)
    else:
        geo_mean = 0

    ratio = float(translation_length) / reference_length

    if ratio > 1.0:
        bp = 1.0
    else:
        bp = math.exp(1 - 1.0 / ratio)

    bleu = geo_mean * bp

    return (bleu, precisions, bp, ratio, translation_length, reference_length)


def bleu_score(
    y_true: List[Union[str, int, List[Union[str, int]]]],
    y_pred: List[Union[str, int, List[Union[str, int]]]],
    ignore_tokens: Optional[List[Union[str, int]]] = None,
):
    if isinstance(y_true, torch.Tensor):
        if y_true.dim() == 2:
            y_true = y_true.unsqueeze(1)
        y_true = convert_tensor_to_strings(y_true)

    if isinstance(y_pred, torch.Tensor) and y_pred.dim() == 2:
        y_pred = convert_tensor_to_strings(y_pred)

    if ignore_tokens is not None:
        ignore_tokens = [str(t) for t in ignore_tokens]

    y_true = remove_strings_ignore_tokens(y_true, ignore_tokens=ignore_tokens)
    y_pred = remove_strings_ignore_tokens(y_pred, ignore_tokens=ignore_tokens)
    return _compute_bleu(y_true, y_pred)[0]



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import torch
from PIL import Image
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from diffusers.utils import numpy_to_pil
from unitorch import is_xformers_available
from unitorch.models.diffusers import (
    ControlNetForImage2ImageGeneration as _ControlNetForImage2ImageGeneration,
)
from unitorch.models.diffusers import ControlNetProcessor

from unitorch.utils import pop_value, nested_dict_value
from unitorch.cli import (
    cached_path,
    add_default_section_for_init,
    add_default_section_for_function,
)
from unitorch.cli.models.diffusers import pretrained_diffusers_infos, load_weight
from unitorch.cli.pipelines import Schedulers


class ControlNetForImage2ImageGenerationPipeline(_ControlNetForImage2ImageGeneration):
    def __init__(
        self,
        config_path: str,
        text_config_path: str,
        vae_config_path: str,
        controlnet_config_path: str,
        scheduler_config_path: str,
        vocab_path: str,
        merge_path: str,
        quant_config_path: Optional[str] = None,
        max_seq_length: Optional[int] = 77,
        pad_token: Optional[str] = "<|endoftext|>",
        weight_path: Optional[Union[str, List[str]]] = None,
        state_dict: Optional[Dict[str, Any]] = None,
        device: Optional[Union[str, int]] = "cpu",
        enable_cpu_offload: Optional[bool] = False,
        enable_xformers: Optional[bool] = False,
    ):
        super().__init__(
            config_path=config_path,
            text_config_path=text_config_path,
            vae_config_path=vae_config_path,
            controlnet_config_path=controlnet_config_path,
            scheduler_config_path=scheduler_config_path,
            quant_config_path=quant_config_path,
        )
        self.processor = ControlNetProcessor(
            vocab_path=vocab_path,
            merge_path=merge_path,
            vae_config_path=vae_config_path,
            max_seq_length=max_seq_length,
            pad_token=pad_token,
        )
        self._device = "cpu" if device == "cpu" else int(device)

        self.from_pretrained(weight_path, state_dict=state_dict)
        self.eval()

        if enable_cpu_offload and self._device != "cpu":
            self.pipeline.enable_model_cpu_offload(self._device)
            self.to(torch.half)
        else:
            self.to(device=self._device)

        if enable_xformers and self._device != "cpu":
            assert is_xformers_available(), "Please install xformers first."
            self.pipeline.enable_xformers_memory_efficient_attention()

    @classmethod
    @add_default_section_for_init("core/pipeline/controlnet/image2image")
    def from_core_configure(cls, config, **kwargs):
        config.set_default_section("core/pipeline/controlnet/image2image")
        pretrained_name = config.getoption(
            "pretrained_name", "stable-v1.5-controlnet-canny"
        )
        pretrain_infos = nested_dict_value(pretrained_diffusers_infos, pretrained_name)

        config_path = config.getoption("config_path", None)
        config_path = pop_value(
            config_path,
            nested_dict_value(pretrain_infos, "unet", "config"),
        )
        config_path = cached_path(config_path)

        text_config_path = config.getoption("text_config_path", None)
        text_config_path = pop_value(
            text_config_path,
            nested_dict_value(pretrain_infos, "text", "config"),
        )
        text_config_path = cached_path(text_config_path)

        vae_config_path = config.getoption("vae_config_path", None)
        vae_config_path = pop_value(
            vae_config_path,
            nested_dict_value(pretrain_infos, "vae", "config"),
        )
        vae_config_path = cached_path(vae_config_path)

        controlnet_config_path = config.getoption("controlnet_config_path", None)
        controlnet_config_path = pop_value(
            controlnet_config_path,
            nested_dict_value(pretrain_infos, "controlnet", "config"),
        )
        controlnet_config_path = cached_path(controlnet_config_path)

        scheduler_config_path = config.getoption("scheduler_config_path", None)
        scheduler_config_path = pop_value(
            scheduler_config_path,
            nested_dict_value(pretrain_infos, "scheduler"),
        )
        scheduler_config_path = cached_path(scheduler_config_path)

        vocab_path = config.getoption("vocab_path", None)
        vocab_path = pop_value(
            vocab_path,
            nested_dict_value(pretrain_infos, "text", "vocab"),
        )
        vocab_path = cached_path(vocab_path)

        merge_path = config.getoption("merge_path", None)
        merge_path = pop_value(
            merge_path,
            nested_dict_value(pretrain_infos, "text", "merge"),
        )
        merge_path = cached_path(merge_path)

        quant_config_path = config.getoption("quant_config_path", None)
        if quant_config_path is not None:
            quant_config_path = cached_path(quant_config_path)

        max_seq_length = config.getoption("max_seq_length", 77)
        pad_token = config.getoption("pad_token", "<|endoftext|>")
        weight_path = config.getoption("pretrained_weight_path", None)
        device = config.getoption("device", "cpu")
        enable_cpu_offload = config.getoption("enable_cpu_offload", True)
        enable_xformers = config.getoption("enable_xformers", True)

        state_dict = None
        if weight_path is None and pretrain_infos is not None:
            state_dict = [
                load_weight(nested_dict_value(pretrain_infos, "unet", "weight")),
                load_weight(nested_dict_value(pretrain_infos, "text", "weight")),
                load_weight(nested_dict_value(pretrain_infos, "vae", "weight")),
                load_weight(
                    nested_dict_value(pretrain_infos, "controlnet", "weight"),
                    prefix_keys={"": "controlnet."},
                ),
            ]

        inst = cls(
            config_path=config_path,
            text_config_path=text_config_path,
            vae_config_path=vae_config_path,
            controlnet_config_path=controlnet_config_path,
            scheduler_config_path=scheduler_config_path,
            vocab_path=vocab_path,
            merge_path=merge_path,
            quant_config_path=quant_config_path,
            pad_token=pad_token,
            max_seq_length=max_seq_length,
            weight_path=weight_path,
            state_dict=state_dict,
            device=device,
            enable_cpu_offload=enable_cpu_offload,
            enable_xformers=enable_xformers,
        )
        return inst

    @torch.no_grad()
    @add_default_section_for_function("core/pipeline/controlnet/image2image")
    def __call__(
        self,
        text: str,
        image: Union[Image.Image, str],
        condition_image: Union[Image.Image, str],
        neg_text: Optional[str] = "",
        guidance_scale: Optional[float] = 7.5,
        controlnet_conditioning_scale: Optional[float] = 1.0,
        num_timesteps: Optional[int] = 50,
        seed: Optional[int] = 1123,
        scheduler: Optional[str] = None,
        freeu_params: Optional[Tuple[float, float, float, float]] = (
            0.9,
            0.2,
            1.2,
            1.4,
        ),
    ):
        inputs = self.processor.image2image_inputs(
            image=image,
            condition_image=condition_image,
            prompt=text,
            negative_prompt=neg_text,
        )
        inputs = {k: v.unsqueeze(0) if v is not None else v for k, v in inputs.items()}
        inputs = {
            k: v.to(device=self._device) if v is not None else v
            for k, v in inputs.items()
        }
        assert scheduler is None or scheduler in Schedulers
        if scheduler is not None:
            self.scheduler = Schedulers.get(scheduler).from_config(
                self.scheduler.config
            )
        self.scheduler.set_timesteps(num_inference_steps=num_timesteps)
        self.pipeline.enable_freeu(*freeu_params)
        self.seed = seed
        outputs = self.generate(
            **inputs,
            guidance_scale=guidance_scale,
            controlnet_conditioning_scale=controlnet_conditioning_scale,
        )
        images = numpy_to_pil(outputs.images.cpu().numpy())
        return images[0]



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import re
import torch
import numpy as np
from PIL import Image
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from unitorch.models.sam import (
    SamForSegmentation as _SamForSegmentation,
)
from unitorch.models.sam import SamProcessor
from unitorch.utils import pop_value, nested_dict_value
from unitorch.cli import (
    cached_path,
    add_default_section_for_init,
    add_default_section_for_function,
)
from unitorch.cli.models.sam import pretrained_sam_infos


class SamPipeline(_SamForSegmentation):
    def __init__(
        self,
        config_path: str,
        vision_config_path: str,
        weight_path: Optional[Union[str, List[str]]] = None,
        state_dict: Optional[Dict[str, Any]] = None,
        device: Optional[Union[str, int]] = "cpu",
    ):
        super().__init__(
            config_path=config_path,
        )
        self.processor = SamProcessor(
            vision_config_path=vision_config_path,
        )
        self._device = "cpu" if device == "cpu" else int(device)

        self.from_pretrained(weight_path, state_dict=state_dict)
        self.to(device=self._device)
        self.eval()

    @classmethod
    @add_default_section_for_init("core/pipeline/sam")
    def from_core_configure(cls, config, **kwargs):
        config.set_default_section("core/pipeline/sam")
        pretrained_name = config.getoption("pretrained_name", "default-sam")

        config_path = config.getoption("config_path", None)
        config_path = pop_value(
            config_path,
            nested_dict_value(pretrained_sam_infos, pretrained_name, "config"),
        )
        config_path = cached_path(config_path)

        vision_config_path = config.getoption("vision_config_path", None)
        vision_config_path = pop_value(
            vision_config_path,
            nested_dict_value(pretrained_sam_infos, pretrained_name, "vision_config"),
        )
        vision_config_path = cached_path(vision_config_path)

        device = config.getoption("device", "cpu")
        pretrained_weight_path = config.getoption("pretrained_weight_path", None)
        weight_path = pop_value(
            pretrained_weight_path,
            nested_dict_value(pretrained_sam_infos, pretrained_name, "weight"),
            check_none=False,
        )

        inst = cls(
            config_path,
            vision_config_path,
            weight_path=weight_path,
            device=device,
        )

        return inst

    @torch.no_grad()
    @add_default_section_for_function("core/pipeline/sam")
    def __call__(
        self,
        image: Union[Image.Image, str],
        points: Optional[List[Tuple[int, int]]] = None,
        boxes: Optional[List[Tuple[int, int, int, int]]] = None,
        mask_threshold: Optional[float] = 0.1,
    ):
        inputs = self.processor.vision_processor(image)
        pixel_values, original_sizes, reshaped_input_sizes = (
            inputs["pixel_values"],
            inputs["original_sizes"],
            inputs["reshaped_input_sizes"],
        )
        pixel_values = [torch.from_numpy(pixel_value) for pixel_value in pixel_values]
        pixel_values = torch.stack(pixel_values).to(self._device)
        input_points = (
            torch.Tensor([[points]]).to(self._device) if points is not None else None
        )
        input_boxes = (
            torch.Tensor([boxes]).to(self._device) if boxes is not None else None
        )
        if input_points is not None:
            input_points[:, :, :, 1] = (
                input_points[:, :, :, 1]
                / original_sizes[0][0]
                * reshaped_input_sizes[0][0]
            )
            input_points[:, :, :, 0] = (
                input_points[:, :, :, 0]
                / original_sizes[0][1]
                * reshaped_input_sizes[0][1]
            )
        outputs = self.segment(
            pixel_values,
            input_points=input_points,
            input_boxes=input_boxes,
        )
        processed_masks = self.processor.vision_processor.post_process_masks(
            outputs.masks,
            original_sizes,
            reshaped_input_sizes,
            mask_threshold=mask_threshold,
            binarize=True,
        )[0]
        if len(processed_masks) == 0:
            return None
        first_mask = processed_masks[0, 0].permute(0, 1)
        first_mask = first_mask.cpu().to(torch.uint8) * 255
        mask_image = Image.fromarray(np.array(first_mask))
        return mask_image



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import io
import torch
import gc
import gradio as gr
from typing import List, Tuple
from PIL import Image, ImageDraw
from unitorch.cli import CoreConfigureParser, GenericWebUI
from unitorch.cli import register_webui
from unitorch.cli.pipelines.sam import SamPipeline


@register_webui("core/webui/sam")
class SamWebUI(GenericWebUI):
    supported_pretrained_names = ["sam-vit-base", "sam-vit-large", "sam-vit-huge"]

    def __init__(self, config: CoreConfigureParser):
        self.config = config
        self._pipe = None if not hasattr(self, "_pipe") else self._pipe
        self._status = "stopped" if self._pipe is None else "running"
        if len(self.supported_pretrained_names) == 0:
            raise ValueError("No supported pretrained models found.")
        self._name = self.supported_pretrained_names[0]
        self._iface = gr.Blocks()
        with self._iface:
            with gr.Row(variant="panel"):
                pretrained_name = gr.Dropdown(
                    self.supported_pretrained_names,
                    value=self._name,
                    label="Pretrain Checkpoint Name",
                )
                status = gr.Textbox(label="Model Status", value=self._status)
                click_start = gr.Button(value="Start")
                click_stop = gr.Button(value="Stop")
                click_start.click(
                    self.start, inputs=[pretrained_name], outputs=[status]
                )
                click_stop.click(self.stop, outputs=[status])

            with gr.Row(variant="panel"):
                origin_input_image = gr.State(None)
                click_points = gr.State([])
                with gr.Column():
                    input_image = gr.Image(
                        type="pil", label="Input Image", interactive=True
                    )
                    submit = gr.Button(value="Submit")
                image = gr.Image(type="pil", label="Output Image")

                input_image.upload(
                    lambda image: image.copy() if image is not None else None,
                    inputs=[input_image],
                    outputs=[origin_input_image],
                )
                input_image.select(
                    self.add_click_points,
                    inputs=[origin_input_image, click_points],
                    outputs=[input_image, click_points],
                )
                submit.click(
                    self.serve,
                    inputs=[origin_input_image, click_points],
                    outputs=[image],
                )

            self._iface.load(
                fn=lambda: gr.update(value=self._name), outputs=[pretrained_name]
            )
            self._iface.load(fn=lambda: gr.update(value=self._status), outputs=[status])

    @property
    def name(self):
        return "SAM"

    @property
    def iface(self):
        return self._iface

    @property
    def status(self):
        return self._status == "running"

    def start(self, pretrained_name, **kwargs):
        if self._status == "running":
            self.stop()
        self.config.set("core/pipeline/sam", "pretrained_name", pretrained_name)
        self._name = pretrained_name
        self._pipe = SamPipeline.from_core_configure(self.config)
        self._status = "running"
        return self._status

    def stop(self, **kwargs):
        self._pipe.to("cpu")
        del self._pipe
        gc.collect()
        torch.cuda.empty_cache()
        self._pipe = None if not hasattr(self, "_pipe") else self._pipe
        self._status = "stopped" if self._pipe is None else "running"
        return self._status

    def add_click_points(self, image, click_points, evt: gr.SelectData):
        x, y = evt.index[0], evt.index[1]
        click_points = click_points + [(x, y)]
        new_image = image.copy()
        draw = ImageDraw.Draw(new_image)
        point_color = (255, 0, 0)
        radius = 10
        for point in click_points:
            x, y = point
            draw.ellipse(
                (x - radius, y - radius, x + radius, y + radius), fill=point_color
            )

        return new_image, click_points

    def serve(
        self,
        image: Image.Image,
        click_points: List[Tuple[int, int]] = [(0, 0)],
    ):
        assert self._pipe is not None
        result = self._pipe(image, points=click_points)
        return result



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

__hf_hub_stable_v1_5_dict__ = lambda name: {
    "unet": {
        "config": f"https://huggingface.co/{name}/resolve/main/unet/config.json",
        "weight": f"https://huggingface.co/{name}/resolve/main/unet/diffusion_pytorch_model.bin",
    },
    "text": {
        "config": f"https://huggingface.co/{name}/resolve/main/text_encoder/config.json",
        "vocab": f"https://huggingface.co/{name}/resolve/main/tokenizer/vocab.json",
        "merge": f"https://huggingface.co/{name}/resolve/main/tokenizer/merges.txt",
        "weight": f"https://huggingface.co/{name}/resolve/main/text_encoder/pytorch_model.bin",
    },
    "vae": {
        "config": f"https://huggingface.co/{name}/resolve/main/vae/config.json",
        "weight": f"https://huggingface.co/{name}/resolve/main/vae/diffusion_pytorch_model.bin",
    },
    "scheduler": f"https://huggingface.co/{name}/resolve/main/scheduler/scheduler_config.json",
}

__hf_hub_stable_v1_5_safetensors_dict__ = lambda name: {
    "unet": {
        "config": f"https://huggingface.co/{name}/resolve/main/unet/config.json",
        "weight": f"https://huggingface.co/{name}/resolve/main/unet/diffusion_pytorch_model.safetensors",
    },
    "text": {
        "config": f"https://huggingface.co/{name}/resolve/main/text_encoder/config.json",
        "vocab": f"https://huggingface.co/{name}/resolve/main/tokenizer/vocab.json",
        "merge": f"https://huggingface.co/{name}/resolve/main/tokenizer/merges.txt",
        "weight": f"https://huggingface.co/{name}/resolve/main/text_encoder/model.safetensors",
    },
    "vae": {
        "config": f"https://huggingface.co/{name}/resolve/main/vae/config.json",
        "weight": f"https://huggingface.co/{name}/resolve/main/vae/diffusion_pytorch_model.safetensors",
    },
    "scheduler": f"https://huggingface.co/{name}/resolve/main/scheduler/scheduler_config.json",
}

__hf_hub_stable_v2_dict__ = __hf_hub_stable_v1_5_dict__
__hf_hub_stable_v2_safetensors_dict__ = __hf_hub_stable_v1_5_safetensors_dict__
__hf_hub_stable_v2_1_dict__ = __hf_hub_stable_v1_5_dict__
__hf_hub_stable_v2_1_safetensors_dict__ = __hf_hub_stable_v1_5_safetensors_dict__

__hf_hub_stable_xl_dict__ = lambda name: {
    "unet": {
        "config": f"https://huggingface.co/{name}/resolve/main/unet/config.json",
        "weight": f"https://huggingface.co/{name}/resolve/main/unet/diffusion_pytorch_model.bin",
    },
    "text": {
        "config": f"https://huggingface.co/{name}/resolve/main/text_encoder/config.json",
        "vocab": f"https://huggingface.co/{name}/resolve/main/tokenizer/vocab.json",
        "merge": f"https://huggingface.co/{name}/resolve/main/tokenizer/merges.txt",
        "weight": f"https://huggingface.co/{name}/resolve/main/text_encoder/pytorch_model.bin",
    },
    "text2": {
        "config": f"https://huggingface.co/{name}/resolve/main/text_encoder_2/config.json",
        "vocab": f"https://huggingface.co/{name}/resolve/main/tokenizer_2/vocab.json",
        "merge": f"https://huggingface.co/{name}/resolve/main/tokenizer_2/merges.txt",
        "weight": f"https://huggingface.co/{name}/resolve/main/text_encoder_2/pytorch_model.bin",
    },
    "vae": {
        "config": f"https://huggingface.co/{name}/resolve/main/vae/config.json",
        "weight": f"https://huggingface.co/{name}/resolve/main/vae/diffusion_pytorch_model.bin",
    },
    "scheduler": f"https://huggingface.co/{name}/resolve/main/scheduler/scheduler_config.json",
}

__hf_hub_stable_xl_safetensors_dict__ = lambda name: {
    "unet": {
        "config": f"https://huggingface.co/{name}/resolve/main/unet/config.json",
        "weight": f"https://huggingface.co/{name}/resolve/main/unet/diffusion_pytorch_model.fp16.safetensors",
    },
    "text": {
        "config": f"https://huggingface.co/{name}/resolve/main/text_encoder/config.json",
        "vocab": f"https://huggingface.co/{name}/resolve/main/tokenizer/vocab.json",
        "merge": f"https://huggingface.co/{name}/resolve/main/tokenizer/merges.txt",
        "weight": f"https://huggingface.co/{name}/resolve/main/text_encoder/model.fp16.safetensors",
    },
    "text2": {
        "config": f"https://huggingface.co/{name}/resolve/main/text_encoder_2/config.json",
        "vocab": f"https://huggingface.co/{name}/resolve/main/tokenizer_2/vocab.json",
        "merge": f"https://huggingface.co/{name}/resolve/main/tokenizer_2/merges.txt",
        "weight": f"https://huggingface.co/{name}/resolve/main/text_encoder_2/model.fp16.safetensors",
    },
    "vae": {
        "config": f"https://huggingface.co/{name}/resolve/main/vae/config.json",
        "weight": f"https://huggingface.co/{name}/resolve/main/vae/diffusion_pytorch_model.fp16.safetensors",
    },
    "scheduler": f"https://huggingface.co/{name}/resolve/main/scheduler/scheduler_config.json",
}

__hf_hub_controlnet_dict__ = lambda name: {
    "controlnet": {
        "config": f"https://huggingface.co/{name}/resolve/main/config.json",
        "weight": f"https://huggingface.co/{name}/resolve/main/diffusion_pytorch_model.bin",
    }
}

__hf_hub_controlnet_safetensors_dict__ = lambda name: {
    "controlnet": {
        "config": f"https://huggingface.co/{name}/resolve/main/config.json",
        "weight": f"https://huggingface.co/{name}/resolve/main/diffusion_pytorch_model.safetensors",
    }
}

__hf_hub_stable_v1_5_controlnet_dict__ = (
    lambda name, base="runwayml/stable-diffusion-v1-5": {
        **__hf_hub_stable_v1_5_dict__(base),
        **__hf_hub_controlnet_dict__(name),
    }
)

__hf_hub_stable_v1_5_safetensors_controlnet_dict__ = (
    lambda name, base="runwayml/stable-diffusion-v1-5": {
        **__hf_hub_stable_v1_5_safetensors_dict__(base),
        **__hf_hub_controlnet_dict__(name),
    }
)

__hf_hub_stable_xl_controlnet_dict__ = (
    lambda name, base="stabilityai/stable-diffusion-xl-base-1.0": {
        **__hf_hub_stable_xl_dict__(base),
        **__hf_hub_controlnet_dict__(name),
    }
)

__hf_hub_stable_xl_safetensors_controlnet_dict__ = (
    lambda name, base="stabilityai/stable-diffusion-xl-base-1.0": {
        **__hf_hub_stable_xl_safetensors_dict__(base),
        **__hf_hub_controlnet_dict__(name),
    }
)

pretrained_stable_infos = {
    "stable-v1.5": __hf_hub_stable_v1_5_dict__("runwayml/stable-diffusion-v1-5"),
    "stable-v1.5-realistic-v5.1-no-vae": __hf_hub_stable_v1_5_safetensors_dict__(
        "SG161222/Realistic_Vision_V5.1_noVAE"
    ),
    "stable-v1.5-realistic-v5.1": __hf_hub_stable_v1_5_dict__(
        "stablediffusionapi/realistic-vision-v51"
    ),
    "stable-v1.5-cyber-realistic": __hf_hub_stable_v1_5_safetensors_dict__(
        "Yntec/CyberRealistic"
    ),
    "stable-v1.5-majicmix-realistic-v6": __hf_hub_stable_v1_5_dict__(
        "digiplay/majicMIX_realistic_v6"
    ),
    "stable-v1.5-chilloutmix": __hf_hub_stable_v1_5_dict__(
        "emilianJR/chilloutmix_NiPrunedFp32Fix"
    ),
    "stable-v1.5-dreamshaper-8": __hf_hub_stable_v1_5_safetensors_dict__(
        "Lykon/dreamshaper-8"
    ),
    "stable-v1.5-nitrosocke-ghibli": __hf_hub_stable_v1_5_dict__(
        "nitrosocke/Ghibli-Diffusion"
    ),
    "stable-v1.5-inpainting": __hf_hub_stable_v1_5_dict__(
        "runwayml/stable-diffusion-inpainting"
    ),
    "stable-v1.5-dreamshaper-8-inpainting": __hf_hub_stable_v1_5_safetensors_dict__(
        "Lykon/dreamshaper-8-inpainting"
    ),
    "stable-v1.5-deliberate-v2-inpainting": __hf_hub_stable_v1_5_dict__(
        "5w4n/deliberate-v2-inpainting"
    ),
    "stable-v1.5-dreambooth-cat-toy": __hf_hub_stable_v1_5_dict__(
        "sd-dreambooth-library/cat-toy"
    ),
    "stable-v2": __hf_hub_stable_v2_dict__("stabilityai/stable-diffusion-2"),
    "stable-v2.1": __hf_hub_stable_v2_1_dict__("stabilityai/stable-diffusion-2-1"),
    "stable-xl-base": __hf_hub_stable_xl_safetensors_dict__(
        "stabilityai/stable-diffusion-xl-base-1.0"
    ),
    "stable-xl-dreamshaper": __hf_hub_stable_xl_safetensors_dict__(
        "Lykon/dreamshaper-xl-1-0"
    ),
    "stable-xl-dreamshaper-fp16": {
        **__hf_hub_stable_xl_safetensors_dict__("Lykon/dreamshaper-xl-1-0"),
        **{
            "vae": {
                "config": "https://huggingface.co/madebyollin/sdxl-vae-fp16-fix/resolve/main/config.json",
                "weight": "https://huggingface.co/madebyollin/sdxl-vae-fp16-fix/resolve/main/diffusion_pytorch_model.safetensors",
            },
        },
    },
    "stable-turbo-xl": __hf_hub_stable_xl_safetensors_dict__("stabilityai/sdxl-turbo"),
    "stable-xl-realism-v30": __hf_hub_stable_xl_dict__(
        "stablediffusionapi/realism-engine-sdxl-v30"
    ),
    "stable-xl-opendalle-v1.1": __hf_hub_stable_xl_safetensors_dict__(
        "dataautogpt3/OpenDalleV1.1"
    ),
    "stable-xl-realvis-v3.0": __hf_hub_stable_xl_safetensors_dict__(
        "SG161222/RealVisXL_V3.0"
    ),
    "stable-xl-juggernaut-v8": __hf_hub_stable_xl_dict__(
        "stablediffusionapi/juggernaut-xl-v8"
    ),
    "stable-xl-playground-v2-aesthetic": __hf_hub_stable_xl_safetensors_dict__(
        "playgroundai/playground-v2-1024px-aesthetic"
    ),
}

pretrained_stable_extensions_infos = {
    "stable-v1.5-controlnet-canny": __hf_hub_controlnet_dict__(
        "lllyasviel/sd-controlnet-canny"
    ),
    "stable-v1.5-controlnet-openpose": __hf_hub_controlnet_dict__(
        "lllyasviel/sd-controlnet-openpose"
    ),
    "stable-v1.5-controlnet-softedge": __hf_hub_controlnet_dict__(
        "lllyasviel/control_v11p_sd15_softedge",
    ),
    "stable-v1.5-controlnet-inpainting": __hf_hub_controlnet_dict__(
        "lllyasviel/control_v11p_sd15_inpaint"
    ),
    "stable-v1.5-animate-v1.5": {
        "motion": {
            "config": f"https://huggingface.co/guoyww/animatediff-motion-adapter-v1-5/resolve/main/config.json",
            "weight": f"https://huggingface.co/guoyww/animatediff-motion-adapter-v1-5/resolve/main/diffusion_pytorch_model.safetensors",
        },
    },
    "stable-v1.5-animate-v1.5.2": {
        "motion": {
            "config": f"https://huggingface.co/guoyww/animatediff-motion-adapter-v1-5-2/resolve/main/config.json",
            "weight": f"https://huggingface.co/guoyww/animatediff-motion-adapter-v1-5-2/resolve/main/diffusion_pytorch_model.safetensors",
        },
    },
    "stable-v1.5-animate-v1.5.2-zoom-in": {
        "motion": {
            "config": f"https://huggingface.co/guoyww/animatediff-motion-adapter-v1-5-2/resolve/main/config.json",
            "weight": f"https://huggingface.co/fuliucansheng/diffusers/resolve/main/animatediff/diffusion_pytorch_model.zoom_in.safetensors",
        },
    },
    "stable-v1.5-animate-v1.5.2-zoom-out": {
        "motion": {
            "config": f"https://huggingface.co/guoyww/animatediff-motion-adapter-v1-5-2/resolve/main/config.json",
            "weight": f"https://huggingface.co/fuliucansheng/diffusers/resolve/main/animatediff/diffusion_pytorch_model.zoom_out.safetensors",
        },
    },
    "stable-xl-controlnet-canny": __hf_hub_controlnet_dict__(
        "diffusers/controlnet-canny-sdxl-1.0"
    ),
    "stable-xl-controlnet-softedge-dexined": __hf_hub_controlnet_dict__(
        "SargeZT/controlnet-sd-xl-1.0-softedge-dexined"
    ),
    "stable-xl-controlnet-depth-small": __hf_hub_controlnet_dict__(
        "diffusers/controlnet-depth-sdxl-1.0-small"
    ),
    "stable-xl-t2i-adapter-canny": __hf_hub_controlnet_safetensors_dict__(
        "TencentARC/t2i-adapter-canny-sdxl-1.0"
    ),
    "stable-xl-t2i-adapter-sketch": __hf_hub_controlnet_safetensors_dict__(
        "TencentARC/t2i-adapter-sketch-sdxl-1.0"
    ),
    "stable-xl-t2i-adapter-openpose": __hf_hub_controlnet_safetensors_dict__(
        "TencentARC/t2i-adapter-openpose-sdxl-1.0"
    ),
    "stable-xl-refiner-1.0": {
        "refiner_unet": {
            "config": f"https://huggingface.co/stabilityai/stable-diffusion-xl-refiner-1.0/resolve/main/unet/config.json",
            "weight": f"https://huggingface.co/stabilityai/stable-diffusion-xl-refiner-1.0/resolve/main/unet/diffusion_pytorch_model.fp16.safetensors",
        },
        "refiner_text2": {
            "config": f"https://huggingface.co/stabilityai/stable-diffusion-xl-refiner-1.0/resolve/main/text_encoder_2/config.json",
            "vocab": f"https://huggingface.co/stabilityai/stable-diffusion-xl-refiner-1.0/resolve/main/tokenizer_2/vocab.json",
            "merge": f"https://huggingface.co/stabilityai/stable-diffusion-xl-refiner-1.0/resolve/main/tokenizer_2/merges.txt",
            "weight": f"https://huggingface.co/stabilityai/stable-diffusion-xl-refiner-1.0/resolve/main/text_encoder_2/model.fp16.safetensors",
        },
        "refiner_vae": {
            "config": f"https://huggingface.co/stabilityai/stable-diffusion-xl-refiner-1.0/resolve/main/vae/config.json",
            "weight": f"https://huggingface.co/stabilityai/stable-diffusion-xl-refiner-1.0/resolve/main/vae/diffusion_pytorch_model.fp16.safetensors",
        },
        "refiner_scheduler": f"https://huggingface.co/stabilityai/stable-diffusion-xl-refiner-1.0/resolve/main/scheduler/scheduler_config.json",
    },
}

stable_version = lambda name: "-".join(name.split("-")[:2])
stable_name = lambda name: "-".join(name.split("-")[2:])
stable_combine_name = (
    lambda base, *names: f"{base}-{'-'.join(stable_name(n) for n in names)}"
)

pretrained_diffusers_infos = {
    **pretrained_stable_infos,
    **{
        stable_combine_name(base, name): {**dict1, **dict2}
        for base, dict1 in pretrained_stable_infos.items()
        for name, dict2 in pretrained_stable_extensions_infos.items()
        if stable_version(name) == stable_version(base)
    },
    "stable-v1.5-x4-upscaler": __hf_hub_stable_v1_5_dict__(
        "stabilityai/stable-diffusion-x4-upscaler"
    ),
    "stable-v1.5-blipdiffusion": {
        **__hf_hub_stable_v1_5_dict__("salesforce/blipdiffusion"),
        **{
            "qformer": {
                "config": f"https://huggingface.co/salesforce/blipdiffusion/resolve/main/qformer/config.json",
                "weight": f"https://huggingface.co/salesforce/blipdiffusion/resolve/main/qformer/pytorch_model.bin",
            },
            "vision_encoder": {
                "config": f"https://huggingface.co/salesforce/blipdiffusion/resolve/main/vision_encoder/config.json",
                "weight": f"https://huggingface.co/salesforce/blipdiffusion/resolve/main/vision_encoder/pytorch_model.bin",
            },
            "vision_config": "https://huggingface.co/salesforce/blipdiffusion/resolve/main/image_processor/preprocessor_config.json",
        },
    },
    "stable-v1.5-blipdiffusion-controlnet": {
        **__hf_hub_stable_v1_5_dict__("salesforce/blipdiffusion-controlnet"),
        **{
            "qformer": {
                "config": f"https://huggingface.co/salesforce/blipdiffusion-controlnet/resolve/main/qformer/config.json",
                "weight": f"https://huggingface.co/salesforce/blipdiffusion-controlnet/resolve/main/qformer/pytorch_model.bin",
            },
            "vision_encoder": {
                "config": f"https://huggingface.co/salesforce/blipdiffusion-controlnet/resolve/main/vision_encoder/config.json",
                "weight": f"https://huggingface.co/salesforce/blipdiffusion-controlnet/resolve/main/vision_encoder/pytorch_model.bin",
            },
            "vision_config": "https://huggingface.co/salesforce/blipdiffusion-controlnet/resolve/main/image_processor/preprocessor_config.json",
            "controlnet": {
                "config": f"https://huggingface.co/salesforce/blipdiffusion-controlnet/resolve/main/controlnet/config.json",
                "weight": f"https://huggingface.co/salesforce/blipdiffusion-controlnet/resolve/main/controlnet/diffusion_pytorch_model.bin",
            },
        },
    },
}

from unitorch.cli.models.diffusion_utils import load_weight

from unitorch.cli.models.diffusers.modeling_stable import (
    StableForText2ImageGeneration,
    StableForImage2ImageGeneration,
    StableForImageInpainting,
    StableForImageResolution,
)
from unitorch.cli.models.diffusers.modeling_stable_xl import (
    StableXLForText2ImageGeneration,
    StableXLForImage2ImageGeneration,
    StableXLForImageInpainting,
)
from unitorch.cli.models.diffusers.modeling_stable_xl_refiner import (
    StableXLRefinerForText2ImageGeneration,
)
from unitorch.cli.models.diffusers.modeling_dreambooth import (
    DreamboothForText2ImageGeneration,
)
from unitorch.cli.models.diffusers.modeling_dreambooth_xl import (
    DreamboothXLForText2ImageGeneration,
)
from unitorch.cli.models.diffusers.modeling_controlnet import (
    ControlNetForText2ImageGeneration,
    ControlNetForImage2ImageGeneration,
    ControlNetForImageInpainting,
)
from unitorch.cli.models.diffusers.modeling_controlnet_xl import (
    ControlNetXLForText2ImageGeneration,
    ControlNetXLForImage2ImageGeneration,
    ControlNetXLForImageInpainting,
)
from unitorch.cli.models.diffusers.modeling_blip2 import (
    Blip2ForText2ImageGeneration,
)

from unitorch.cli.models.diffusers.processing_stable import StableProcessor
from unitorch.cli.models.diffusers.processing_stable_xl import StableXLProcessor
from unitorch.cli.models.diffusers.processing_stable_xl_refiner import (
    StableXLRefinerProcessor,
)
from unitorch.cli.models.diffusers.processing_dreambooth import DreamboothProcessor
from unitorch.cli.models.diffusers.processing_dreambooth_xl import DreamboothXLProcessor
from unitorch.cli.models.diffusers.processing_controlnet import ControlNetProcessor
from unitorch.cli.models.diffusers.processing_controlnet_xl import ControlNetXLProcessor
from unitorch.cli.models.diffusers.processing_blip2 import Blip2Processor

import unitorch.cli.models.diffusers.modeling_animate



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from unitorch.models.beit.modeling import (
    BeitForImageClassification,
)
from unitorch.models.beit.processing import BeitProcessor



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import torch
import json
import numpy as np
from PIL import Image
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from transformers import CLIPTokenizer
from torchvision.transforms import (
    Resize,
    CenterCrop,
    RandomCrop,
    Lambda,
    ToTensor,
    Normalize,
    Compose,
    RandomHorizontalFlip,
)
from diffusers.image_processor import VaeImageProcessor
from unitorch.models import HfTextClassificationProcessor, GenericOutputs


class StableProcessor(HfTextClassificationProcessor):
    def __init__(
        self,
        vocab_path: str,
        merge_path: str,
        vision_config_path: Optional[str] = None,
        vae_config_path: Optional[str] = None,
        max_seq_length: Optional[int] = 77,
        position_start_id: Optional[int] = 0,
        pad_token: Optional[str] = "<|endoftext|>",
        image_size: Optional[int] = 512,
        center_crop: Optional[bool] = False,
        random_flip: Optional[bool] = False,
    ):
        tokenizer = CLIPTokenizer(
            vocab_file=vocab_path,
            merges_file=merge_path,
        )

        tokenizer.cls_token = tokenizer.bos_token
        tokenizer.sep_token = tokenizer.eos_token
        tokenizer.pad_token = pad_token

        HfTextClassificationProcessor.__init__(
            self,
            tokenizer=tokenizer,
            max_seq_length=max_seq_length,
            position_start_id=position_start_id,
        )

        self.vision_processor = Compose(
            [
                Resize(image_size),
                CenterCrop(image_size) if center_crop else RandomCrop(image_size),
                RandomHorizontalFlip() if random_flip else Lambda(lambda x: x),
                ToTensor(),
                Normalize([0.5], [0.5]),
            ]
        )

        if vae_config_path is not None:
            vae_config_dict = json.load(open(vae_config_path))
            vae_scale_factor = 2 ** (
                len(vae_config_dict.get("block_out_channels", [])) - 1
            )
            self.vae_image_processor = VaeImageProcessor(
                vae_scale_factor=vae_scale_factor
            )
        else:
            self.vae_image_processor = None

    def text2image(
        self,
        prompt: str,
        image: Union[Image.Image, str],
        max_seq_length: Optional[int] = None,
    ):
        if isinstance(image, str):
            image = Image.open(image)

        image = image.convert("RGB")

        pixel_values = self.vision_processor(image)

        prompt_outputs = self.classification(prompt, max_seq_length=max_seq_length)

        return GenericOutputs(
            pixel_values=pixel_values,
            input_ids=prompt_outputs.input_ids,
            attention_mask=prompt_outputs.attention_mask,
        )

    def text2image_inputs(
        self,
        prompt: str,
        negative_prompt: Optional[str] = "",
        max_seq_length: Optional[int] = None,
    ):
        prompt_outputs = self.classification(prompt, max_seq_length=max_seq_length)
        negative_prompt_outputs = self.classification(
            negative_prompt, max_seq_length=max_seq_length
        )

        return GenericOutputs(
            input_ids=prompt_outputs.input_ids,
            attention_mask=prompt_outputs.attention_mask,
            negative_input_ids=negative_prompt_outputs.input_ids,
            negative_attention_mask=negative_prompt_outputs.attention_mask,
        )

    def image2image_inputs(
        self,
        prompt: str,
        image: Union[Image.Image, str],
        negative_prompt: Optional[str] = "",
        max_seq_length: Optional[int] = None,
    ):
        if isinstance(image, str):
            image = Image.open(image)
        image = image.convert("RGB")

        pixel_values = self.vae_image_processor.preprocess(image)[0]

        text_inputs = self.text2image_inputs(
            prompt,
            negative_prompt=negative_prompt,
            max_seq_length=max_seq_length,
        )

        return GenericOutputs(
            pixel_values=pixel_values,
            **text_inputs,
        )

    def inpainting_inputs(
        self,
        prompt: str,
        image: Union[Image.Image, str],
        mask_image: Union[Image.Image, str],
        negative_prompt: Optional[str] = "",
        max_seq_length: Optional[int] = None,
    ):
        if isinstance(image, str):
            image = Image.open(image)
        image = image.convert("RGB")

        if isinstance(mask_image, str):
            mask_image = Image.open(mask_image)
        mask_image = mask_image.convert("L")

        pixel_values = self.vae_image_processor.preprocess(image)[0]
        pixel_masks = self.vae_image_processor.preprocess(mask_image)[0]
        pixel_masks = (pixel_masks + 1) / 2

        text_inputs = self.text2image_inputs(
            prompt,
            negative_prompt=negative_prompt,
            max_seq_length=max_seq_length,
        )

        return GenericOutputs(
            pixel_values=pixel_values,
            pixel_masks=pixel_masks,
            **text_inputs,
        )

    def resolution_inputs(
        self,
        prompt: str,
        image: Union[Image.Image, str],
        negative_prompt: Optional[str] = "",
        max_seq_length: Optional[int] = None,
    ):
        if isinstance(image, str):
            image = Image.open(image)
        image = image.convert("RGB")

        pixel_values = self.vae_image_processor.preprocess(image)[0]

        text_inputs = self.text2image_inputs(
            prompt,
            negative_prompt=negative_prompt,
            max_seq_length=max_seq_length,
        )

        return GenericOutputs(
            pixel_values=pixel_values,
            **text_inputs,
        )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import torch
import hashlib
import numpy as np
import torch.nn as nn
from PIL import Image
from dataclasses import dataclass
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from unitorch.utils import numpy_to_pil
from unitorch.cli import (
    add_default_section_for_init,
    add_default_section_for_function,
    register_process,
)
from unitorch.cli import WriterMixin, WriterOutputs
from unitorch.cli.models.modeling_utils import ListTensorsOutputs, ListTensorsTargets


@dataclass
class SegmentationOutputs(ListTensorsOutputs, WriterMixin):
    masks: Union[torch.Tensor, List[torch.Tensor]]


@dataclass
class SegmentationTargets(ListTensorsTargets):
    targets: Union[torch.Tensor, List[torch.Tensor]]
    sample_weight: Optional[torch.Tensor] = torch.empty(0)


class SegmentationProcessor:
    def __init__(
        self,
        mask_threshold: float = None,
        output_folder: Optional[str] = None,
    ):
        self.mask_threshold = mask_threshold

        self.output_folder = output_folder
        if self.output_folder is not None and not os.path.exists(output_folder):
            os.makedirs(self.output_folder, exist_ok=True)

    @classmethod
    @add_default_section_for_init("core/process/segmentation")
    def from_core_configure(cls, config, **kwargs):
        pass

    def save_image(self, image: Image.Image):
        md5 = hashlib.md5()
        md5.update(image.tobytes())
        name = md5.hexdigest() + ".jpg"
        output_path = f"{self.output_folder}/{name}"
        image.save(output_path)
        return name

    @register_process("core/postprocess/segmentation")
    def _segmentation(
        self,
        outputs: SegmentationOutputs,
    ):
        results = outputs.to_pandas()
        assert results.shape[0] == 0 or results.shape[0] == len(outputs.masks)
        mask0 = outputs.masks[0].numpy()
        if (
            mask0.shape[-1] == 1 or mask0.ndim == 2
        ) and self.mask_threshold is not None:
            results["pixel_class"] = [
                (m.numpy() > self.mask_threshold).astype(np.int16).reshape(-1).tolist()
                for m in outputs.masks
            ]
        else:
            results["pixel_class"] = [
                m.numpy().argmax(-1).reshape(-1).tolist() for m in outputs.masks
            ]
        return WriterOutputs(results)

    @register_process("core/postprocess/segmentation/image")
    def _diffusion_image(
        self,
        outputs: SegmentationOutputs,
    ):
        results = outputs.to_pandas()
        assert results.shape[0] == 0 or results.shape[0] == len(outputs.masks)
        images = (
            outputs.masks.numpy()
            if isinstance(outputs.masks, torch.Tensor)
            else [m.numpy() for m in outputs.masks]
        )
        images = [numpy_to_pil(image) for image in images]
        results["mask_image"] = [self.save_image(image) for image in images]
        return WriterOutputs(results)


def segmentation_model_decorator(cls):
    class SegmentationModel(nn.Module):
        def __init__(self, *args, **kwargs):
            super().__init__()
            if "__segmentation_model__" in kwargs:
                self.model = kwargs.pop("__segmentation_model__")
            else:
                self.model = cls(*args, **kwargs)

            __more_attrs__ = [
                "load_state_dict",
                "state_dict",
                "save_checkpoint",
                "from_checkpoint",
                "from_pretrained",
            ]
            for __more_attr__ in __more_attrs__:
                setattr(self, __more_attr__, getattr(self.model, __more_attr__))

            assert hasattr(self.model, "segment")

        def forward(self, *args, **kwargs):
            if self.training:
                return self.model(*args, **kwargs)
            return self.model.segment(*args, **kwargs)

        @classmethod
        def from_core_configure(_cls, cfg, **kwargs):
            model = cls.from_core_configure(cfg, **kwargs)
            return _cls(__segmentation_model__=model)

    return SegmentationModel



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import torch
from functools import partial
from typing import Any, Callable, Dict, Iterable, List, Optional, Tuple, Union

from transformers import PegasusTokenizer
from unitorch.models import HfTextGenerationProcessor


def get_xpegasus_tokenizer(
    vocab_path,
    special_input_ids=dict(),
):
    tokenizer = PegasusTokenizer(vocab_path)
    for token, _id in special_input_ids.items():
        tokenizer.added_tokens_encoder[token] = _id
        tokenizer.unique_no_split_tokens.append(token)
        tokenizer.added_tokens_decoder[_id] = token
        tokenizer.add_tokens(token, special_tokens=True)
    return tokenizer


class XPegasusProcessor(HfTextGenerationProcessor):
    def __init__(
        self,
        vocab_path: str,
        special_input_ids: Optional[Dict] = dict(),
        max_seq_length: Optional[int] = 128,
        max_gen_seq_length: Optional[int] = 48,
    ):
        tokenizer = get_xpegasus_tokenizer(
            vocab_path,
            special_input_ids=special_input_ids,
        )
        tokenizer.bos_token_id = 0
        tokenizer.bos_token = tokenizer.convert_ids_to_tokens(0)
        tokenizer.sep_token = tokenizer.eos_token
        tokenizer.sep_token_id = tokenizer.eos_token_id
        super().__init__(
            tokenizer=tokenizer,
            max_seq_length=max_seq_length,
            max_gen_seq_length=max_gen_seq_length,
        )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import io
import torch
import gc
import gradio as gr
from PIL import Image, ImageFilter, ImageOps
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from unitorch.cli import CoreConfigureParser, GenericWebUI
from unitorch.cli import register_webui


@register_webui("core/webui/image_utils")
class ImageUtilsWebUI(GenericWebUI):
    def __init__(self, config: CoreConfigureParser):
        self.config = config
        self._status = "running"
        self._iface = gr.Blocks()
        with self._iface:
            # canny
            with gr.Row(variant="panel"):
                with gr.Column():
                    image = gr.Image(type="pil", label="Input Image")
                    height = gr.Slider(512, 1024, value=512, label="Image Height")
                    width = gr.Slider(512, 1024, value=512, label="Image Width")
                    submit = gr.Button(value="Submit")
                canny_image = gr.Image(type="pil", label="Output Canny Image")

                submit.click(
                    self.canny,
                    inputs=[image, height, width],
                    outputs=[canny_image],
                )

            # blend
            with gr.Row(variant="panel"):
                with gr.Column():
                    with gr.Row():
                        image1 = gr.Image(type="pil", label="Input Image1")
                        image2 = gr.Image(type="pil", label="Input Image2")
                    height2 = gr.Slider(512, 1024, value=512, label="Image Height")
                    width2 = gr.Slider(512, 1024, value=512, label="Image Width")
                    alpha = gr.Slider(0, 1, value=0.5, label="Alpha")
                    submit2 = gr.Button(value="Submit")
                blend_image = gr.Image(type="pil", label="Output Blend Image")

                submit2.click(
                    self.blend,
                    inputs=[image1, image2, alpha, height2, width2],
                    outputs=[blend_image],
                )

            # invert
            with gr.Row(variant="panel"):
                with gr.Column():
                    image3 = gr.Image(type="pil", label="Input Image")
                    height3 = gr.Slider(512, 1024, value=512, label="Image Height")
                    width3 = gr.Slider(512, 1024, value=512, label="Image Width")
                    submit3 = gr.Button(value="Submit")
                invert_image = gr.Image(type="pil", label="Output Invert Image")

                submit3.click(
                    self.invert,
                    inputs=[image3, height3, width3],
                    outputs=[invert_image],
                )

    @property
    def name(self):
        return "ImageUtils"

    @property
    def iface(self):
        return self._iface

    @property
    def status(self):
        return self._status == "running"

    def start(self):
        pass

    def stop(self):
        passs

    def canny(
        self,
        image: Image.Image,
        height: Optional[int] = 512,
        width: Optional[int] = 512,
    ):
        image = image.resize((height, width))
        image = image.convert("L")
        image = image.filter(ImageFilter.FIND_EDGES)
        return image

    def blend(
        self,
        image1,
        image2,
        alpha=0.5,
        height: Optional[int] = 512,
        width: Optional[int] = 512,
    ):
        image1 = image1.resize((height, width))
        image2 = image2.resize((height, width))
        image = Image.blend(image1, image2, alpha)
        return image

    def invert(
        self,
        image,
        height: Optional[int] = 512,
        width: Optional[int] = 512,
    ):
        image = image.resize((height, width))
        image = ImageOps.invert(image)
        return image



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from unitorch.utils import pop_value, nested_dict_value
from unitorch.models.bert import BertProcessor as _BertProcessor
from unitorch.cli import (
    cached_path,
    add_default_section_for_init,
    add_default_section_for_function,
    register_process,
)
from unitorch.cli import WriterOutputs
from unitorch.cli.models import (
    TensorsInputs,
    GenerationOutputs,
    GenerationTargets,
)
from unitorch.cli.models.bert import pretrained_bert_infos


class BertProcessor(_BertProcessor):
    """Processor for BERT models."""

    def __init__(
        self,
        vocab_path: str,
        max_seq_length: Optional[int] = 128,
        special_input_ids: Optional[Dict] = dict(),
        do_lower_case: Optional[bool] = True,
        do_basic_tokenize: Optional[bool] = True,
        do_whole_word_mask: Optional[bool] = True,
        masked_lm_prob: Optional[float] = 0.15,
        max_predictions_per_seq: Optional[int] = 20,
    ):
        """
        Initialize BertProcessor.

        Args:
            vocab_path (str): The path to the vocabulary file.
            max_seq_length (int, optional): The maximum sequence length. Defaults to 128.
            special_input_ids (Dict, optional): Special input IDs. Defaults to an empty dictionary.
            do_lower_case (bool, optional): Whether to lower case the input text. Defaults to True.
            do_basic_tokenize (bool, optional): Whether to perform basic tokenization. Defaults to True.
            do_whole_word_mask (bool, optional): Whether to use whole word masking. Defaults to True.
            masked_lm_prob (float, optional): The probability of masking a token for masked language modeling. Defaults to 0.15.
            max_predictions_per_seq (int, optional): The maximum number of masked LM predictions per sequence. Defaults to 20.
        """
        super().__init__(
            vocab_path=vocab_path,
            max_seq_length=max_seq_length,
            special_input_ids=special_input_ids,
            do_lower_case=do_lower_case,
            do_basic_tokenize=do_basic_tokenize,
            do_whole_word_mask=do_whole_word_mask,
            masked_lm_prob=masked_lm_prob,
            max_predictions_per_seq=max_predictions_per_seq,
        )

    @classmethod
    @add_default_section_for_init("core/process/bert")
    def from_core_configure(cls, config, **kwargs):
        """
        Create an instance of BertProcessor from a core configuration.

        Args:
            config: The core configuration.
            **kwargs: Additional keyword arguments.

        Returns:
            BertProcessor: An instance of BertProcessor.
        """
        config.set_default_section("core/process/bert")
        pretrained_name = config.getoption("pretrained_name", "default-bert")
        vocab_path = config.getoption("vocab_path", None)
        vocab_path = pop_value(
            vocab_path,
            nested_dict_value(pretrained_bert_infos, pretrained_name, "vocab"),
        )
        vocab_path = cached_path(vocab_path)

        return {
            "vocab_path": vocab_path,
        }

    @register_process("core/process/bert/classification")
    def _classification(
        self,
        text: str,
        text_pair: Optional[str] = None,
        max_seq_length: Optional[int] = None,
    ):
        """
        Run classification process using the BertProcessor.

        Args:
            text (str): The input text.
            text_pair (str, optional): The second input text for sequence pair classification. Defaults to None.
            max_seq_length (int, optional): The maximum sequence length. Defaults to None.

        Returns:
            TensorsInputs: Inputs for the classification process.
        """
        outputs = super().classification(
            text=text,
            text_pair=text_pair,
            max_seq_length=max_seq_length,
        )
        return TensorsInputs(
            input_ids=outputs.input_ids,
            attention_mask=outputs.attention_mask,
            token_type_ids=outputs.token_type_ids,
            position_ids=outputs.position_ids,
        )

    @register_process("core/process/bert/pretrain")
    def _pretrain(
        self,
        text: str,
        text_pair: str,
        nsp_label: int,
        max_seq_length: Optional[int] = None,
        masked_lm_prob: Optional[float] = None,
        do_whole_word_mask: Optional[bool] = None,
        max_predictions_per_seq: Optional[int] = None,
    ):
        """
        Run pretraining process using the BertProcessor.

        Args:
            text (str): The input text.
            text_pair (str): The second input text for sequence pair pretraining.
            nsp_label (int): The next sentence prediction label.
            max_seq_length (int, optional): The maximum sequence length. Defaults to None.
            masked_lm_prob (float, optional): The probability of masking a token for masked language modeling. Defaults to None.
            do_whole_word_mask (bool, optional): Whether to use whole word masking. Defaults to None.
            max_predictions_per_seq (int, optional): The maximum number of masked LM predictions per sequence. Defaults to None.

        Returns:
            TensorsInputs: Inputs for the pretraining process.
        """
        outputs = super().pretrain(
            text=text,
            text_pair=text_pair,
            nsp_label=nsp_label,
            max_seq_length=max_seq_length,
            masked_lm_prob=masked_lm_prob,
            do_whole_word_mask=do_whole_word_mask,
            max_predictions_per_seq=max_predictions_per_seq,
        )
        return TensorsInputs(
            input_ids=outputs.input_ids,
            attention_mask=outputs.attention_mask,
            token_type_ids=outputs.token_type_ids,
            position_ids=outputs.position_ids,
            nsp_label=outputs.nsp_label,
            mlm_label=outputs.mlm_label,
            mlm_label_mask=outputs.mlm_label_mask,
        )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import torch
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from torch.cuda.amp import autocast
from unitorch.utils import pop_value, nested_dict_value
from unitorch.models.visualbert import (
    VisualBertForClassification as _VisualBertForClassification,
)
from unitorch.models.visualbert import VisualBertForPretrain as _VisualBertForPretrain
from unitorch.cli import (
    cached_path,
    add_default_section_for_init,
    add_default_section_for_function,
    register_model,
)
from unitorch.cli.models import ClassificationOutputs, LossOutputs
from unitorch.cli.models.visualbert import pretrained_visualbert_infos


@register_model("core/model/classification/visualbert")
class VisualBertForClassification(_VisualBertForClassification):
    """VisualBERT for Classification model."""

    def __init__(
        self,
        config_path: str,
        num_classes: Optional[int] = 1,
        gradient_checkpointing: Optional[bool] = False,
    ):
        """
        Initialize VisualBertForClassification.

        Args:
            config_path (str): The path to the model's configuration file.
            num_classes (Optional[int]): The number of classes for classification.
                Defaults to 1.
            gradient_checkpointing (Optional[bool]): Whether to use gradient checkpointing
                to save memory during training. Defaults to False.
        """
        super().__init__(
            config_path=config_path,
            num_classes=num_classes,
            gradient_checkpointing=gradient_checkpointing,
        )

    @classmethod
    @add_default_section_for_init("core/model/classification/visualbert")
    def from_core_configure(cls, config, **kwargs):
        """
        Create an instance of VisualBertForClassification from a core configuration.

        Args:
            config: The core configuration.
            **kwargs: Additional keyword arguments.

        Returns:
            VisualBertForClassification: The initialized VisualBertForClassification instance.
        """
        config.set_default_section("core/model/classification/visualbert")
        pretrained_name = config.getoption("pretrained_name", "default-visualbert")
        config_path = config.getoption("config_path", None)

        config_path = pop_value(
            config_path,
            nested_dict_value(pretrained_visualbert_infos, pretrained_name, "config"),
        )

        config_path = cached_path(config_path)
        gradient_checkpointing = config.getoption("gradient_checkpointing", False)
        num_classes = config.getoption("num_classes", 1)

        inst = cls(config_path, num_classes, gradient_checkpointing)
        pretrained_weight_path = config.getoption("pretrained_weight_path", None)
        weight_path = pop_value(
            pretrained_weight_path,
            nested_dict_value(pretrained_visualbert_infos, pretrained_name, "weight"),
            check_none=False,
        )
        if weight_path is not None:
            weight_path = cached_path(weight_path)
            inst.from_pretrained(weight_path)

        return inst

    @autocast()
    def forward(
        self,
        input_ids: torch.Tensor,
        attention_mask: torch.Tensor,
        token_type_ids: torch.Tensor,
        position_ids: torch.Tensor,
        visual_embeds: torch.Tensor,
        visual_attention_mask: torch.Tensor,
        visual_token_type_ids: torch.Tensor,
    ):
        """
        Forward pass of the VisualBertForClassification model.

        Args:
            input_ids (torch.Tensor): The input token IDs.
            attention_mask (torch.Tensor): The attention mask.
            token_type_ids (torch.Tensor): The token type IDs.
            position_ids (torch.Tensor): The position IDs.
            visual_embeds (torch.Tensor): The visual embeddings.
            visual_attention_mask (torch.Tensor): The visual attention mask.
            visual_token_type_ids (torch.Tensor): The visual token type IDs.

        Returns:
            ClassificationOutputs: The output logits of the model.
        """
        outputs = super().forward(
            input_ids=input_ids,
            attention_mask=attention_mask,
            token_type_ids=token_type_ids,
            position_ids=position_ids,
            visual_embeds=visual_embeds,
            visual_attention_mask=visual_attention_mask,
            visual_token_type_ids=visual_token_type_ids,
        )
        return ClassificationOutputs(outputs=outputs)


@register_model("core/model/pretrain/visualbert")
class VisualBertForPretrain(_VisualBertForPretrain):
    """VisualBERT for Pretraining model."""

    def __init__(
        self,
        config_path: str,
        gradient_checkpointing: Optional[bool] = False,
    ):
        """
        Initialize VisualBertForPretrain.

        Args:
            config_path (str): The path to the model's configuration file.
            gradient_checkpointing (Optional[bool]): Whether to use gradient checkpointing
                to save memory during training. Defaults to False.
        """
        super().__init__(
            config_path=config_path,
            gradient_checkpointing=gradient_checkpointing,
        )

    @classmethod
    @add_default_section_for_init("core/model/pretrain/visualbert")
    def from_core_configure(cls, config, **kwargs):
        """
        Create an instance of VisualBertForPretrain from a core configuration.

        Args:
            config: The core configuration.
            **kwargs: Additional keyword arguments.

        Returns:
            VisualBertForPretrain: The initialized VisualBertForPretrain instance.
        """
        config.set_default_section("core/model/pretrain/visualbert")
        pretrained_name = config.getoption("pretrained_name", "default-visualbert")
        config_path = config.getoption("config_path", None)

        config_path = pop_value(
            config_path,
            nested_dict_value(pretrained_visualbert_infos, pretrained_name, "config"),
        )

        config_path = cached_path(config_path)
        gradient_checkpointing = config.getoption("gradient_checkpointing", False)

        inst = cls(config_path, gradient_checkpointing)
        pretrained_weight_path = config.getoption("pretrained_weight_path", None)
        weight_path = pop_value(
            pretrained_weight_path,
            nested_dict_value(pretrained_visualbert_infos, pretrained_name, "weight"),
            check_none=False,
        )
        if weight_path is not None:
            weight_path = cached_path(weight_path)
            inst.from_pretrained(weight_path)

        return inst

    @autocast()
    def forward(
        self,
        input_ids: torch.Tensor,
        attention_mask: torch.Tensor,
        token_type_ids: torch.Tensor,
        position_ids: torch.Tensor,
        visual_embeds: torch.Tensor,
        visual_attention_mask: torch.Tensor,
        visual_token_type_ids: torch.Tensor,
        nsp_label: torch.Tensor,
        mlm_label: torch.Tensor,
        mlm_label_mask: torch.Tensor,
    ):
        """
        Forward pass of the VisualBertForPretrain model.

        Args:
            input_ids (torch.Tensor): The input token IDs.
            attention_mask (torch.Tensor): The attention mask.
            token_type_ids (torch.Tensor): The token type IDs.
            position_ids (torch.Tensor): The position IDs.
            visual_embeds (torch.Tensor): The visual embeddings.
            visual_attention_mask (torch.Tensor): The visual attention mask.
            visual_token_type_ids (torch.Tensor): The visual token type IDs.
            nsp_label (torch.Tensor): The next sentence prediction label.
            mlm_label (torch.Tensor): The masked language model label.
            mlm_label_mask (torch.Tensor): The masked language model label mask.

        Returns:
            LossOutputs: The output loss of the model.
        """
        outputs = super().forward(
            input_ids=input_ids,
            attention_mask=attention_mask,
            token_type_ids=token_type_ids,
            position_ids=position_ids,
            visual_embeds=visual_embeds,
            visual_attention_mask=visual_attention_mask,
            visual_token_type_ids=visual_token_type_ids,
            nsp_label=nsp_label,
            mlm_label=mlm_label,
            mlm_label_mask=mlm_label_mask,
        )
        return LossOutputs(loss=outputs)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from unitorch import is_diffusers_available

if is_diffusers_available():
    from unitorch.models.diffusers.modeling_stable import (
        StableForText2ImageGeneration,
        StableForImage2ImageGeneration,
        StableForImageInpainting,
        StableForImageResolution,
    )
    from unitorch.models.diffusers.modeling_stable_xl import (
        StableXLForText2ImageGeneration,
        StableXLForImage2ImageGeneration,
        StableXLForImageInpainting,
    )
    from unitorch.models.diffusers.modeling_stable_xl_refiner import (
        StableXLRefinerForText2ImageGeneration,
        StableXLRefinerForImage2ImageGeneration,
        StableXLRefinerForImageInpainting,
    )
    from unitorch.models.diffusers.modeling_dreambooth import (
        DreamboothForText2ImageGeneration,
    )
    from unitorch.models.diffusers.modeling_dreambooth_xl import (
        DreamboothXLForText2ImageGeneration,
    )
    from unitorch.models.diffusers.modeling_controlnet import (
        ControlNetForText2ImageGeneration,
        ControlNetForImage2ImageGeneration,
        ControlNetForImageInpainting,
    )
    from unitorch.models.diffusers.modeling_controlnet_xl import (
        ControlNetXLForText2ImageGeneration,
        ControlNetXLForImage2ImageGeneration,
        ControlNetXLForImageInpainting,
    )
    from unitorch.models.diffusers.modeling_multicontrolnet import (
        MultiControlNetForText2ImageGeneration,
        MultiControlNetForImage2ImageGeneration,
        MultiControlNetForImageInpainting,
    )
    from unitorch.models.diffusers.modeling_animate import (
        AnimateForText2VideoGeneration,
        AnimateForImage2VideoGeneration,
    )
    from unitorch.models.diffusers.modeling_blip2 import (
        Blip2ForText2ImageGeneration,
        Blip2ControlNetForText2ImageGeneration,
    )
    from unitorch.models.diffusers.processing_stable import StableProcessor
    from unitorch.models.diffusers.processing_stable_xl import StableXLProcessor
    from unitorch.models.diffusers.processing_stable_xl_refiner import (
        StableXLRefinerProcessor,
    )
    from unitorch.models.diffusers.processing_dreambooth import DreamboothProcessor
    from unitorch.models.diffusers.processing_dreambooth_xl import DreamboothXLProcessor
    from unitorch.models.diffusers.processing_controlnet import ControlNetProcessor
    from unitorch.models.diffusers.processing_controlnet_xl import ControlNetXLProcessor
    from unitorch.models.diffusers.processing_multicontrolnet import (
        MultiControlNetProcessor,
    )
    from unitorch.models.diffusers.processing_blip2 import Blip2Processor



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import torch
import json
import numpy as np
from PIL import Image
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from transformers import CLIPTokenizer
from torchvision.transforms import (
    Resize,
    CenterCrop,
    RandomCrop,
    ToTensor,
    Normalize,
    Compose,
    RandomHorizontalFlip,
)
from torchvision.transforms.functional import crop
from diffusers.image_processor import VaeImageProcessor
from unitorch.models import HfTextClassificationProcessor, GenericOutputs


class StableXLRefinerProcessor:
    def __init__(
        self,
        vocab1_path: str,
        merge1_path: str,
        vocab2_path: str,
        merge2_path: str,
        refiner_vocab1_path: Optional[str] = None,
        refiner_merge1_path: Optional[str] = None,
        refiner_vocab2_path: Optional[str] = None,
        refiner_merge2_path: Optional[str] = None,
        vae_config_path: Optional[str] = None,
        max_seq_length: Optional[int] = 77,
        position_start_id: Optional[int] = 0,
        pad_token1: Optional[str] = "<|endoftext|>",
        pad_token2: Optional[str] = "!",
        refiner_pad_token1: Optional[str] = "<|endoftext|>",
        refiner_pad_token2: Optional[str] = "!",
        image_size: Optional[int] = 512,
        center_crop: Optional[bool] = False,
    ):
        tokenizer1 = CLIPTokenizer(
            vocab_file=vocab1_path,
            merges_file=merge1_path,
        )

        tokenizer1.cls_token = tokenizer1.bos_token
        tokenizer1.sep_token = tokenizer1.eos_token
        tokenizer1.pad_token = pad_token1

        self.text_processor1 = HfTextClassificationProcessor(
            tokenizer=tokenizer1,
            max_seq_length=max_seq_length,
            position_start_id=position_start_id,
        )

        tokenizer2 = CLIPTokenizer(
            vocab_file=vocab2_path,
            merges_file=merge2_path,
        )

        tokenizer2.cls_token = tokenizer2.bos_token
        tokenizer2.sep_token = tokenizer2.eos_token
        tokenizer2.pad_token = pad_token2

        tokenizer2.pad_token = "!"
        self.text_processor2 = HfTextClassificationProcessor(
            tokenizer=tokenizer2,
            max_seq_length=max_seq_length,
            position_start_id=position_start_id,
        )

        if refiner_vocab1_path is not None and refiner_merge1_path is not None:
            refiner_tokenizer1 = CLIPTokenizer(
                vocab_file=refiner_vocab1_path,
                merges_file=refiner_merge1_path,
            )

            refiner_tokenizer1.cls_token = refiner_tokenizer1.bos_token
            refiner_tokenizer1.sep_token = refiner_tokenizer1.eos_token
            refiner_tokenizer1.pad_token = refiner_pad_token1

            self.refiner_text_processor1 = HfTextClassificationProcessor(
                tokenizer=refiner_tokenizer1,
                max_seq_length=max_seq_length,
                position_start_id=position_start_id,
            )
        else:
            self.refiner_text_processor1 = None

        if refiner_vocab2_path is not None and refiner_merge2_path is not None:
            refiner_tokenizer2 = CLIPTokenizer(
                vocab_file=refiner_vocab2_path,
                merges_file=refiner_merge2_path,
            )

            refiner_tokenizer2.cls_token = refiner_tokenizer2.bos_token
            refiner_tokenizer2.sep_token = refiner_tokenizer2.eos_token
            refiner_tokenizer2.pad_token = refiner_pad_token2

            refiner_tokenizer2.pad_token = "!"
            self.refiner_text_processor2 = HfTextClassificationProcessor(
                tokenizer=refiner_tokenizer2,
                max_seq_length=max_seq_length,
                position_start_id=position_start_id,
            )
        else:
            self.refiner_text_processor2 = None

        self.image_size = image_size
        self.center_crop = center_crop
        self.vision_resize = Resize(image_size)
        self.vision_crop = (
            CenterCrop(image_size) if center_crop else RandomCrop(image_size)
        )
        self.vision_flip = RandomHorizontalFlip(p=1.0)
        self.vision_processor = Compose(
            [
                ToTensor(),
                Normalize([0.5], [0.5]),
            ]
        )

        if vae_config_path is not None:
            vae_config_dict = json.load(open(vae_config_path))
            vae_scale_factor = 2 ** (
                len(vae_config_dict.get("block_out_channels", [])) - 1
            )
            self.vae_image_processor = VaeImageProcessor(
                vae_scale_factor=vae_scale_factor
            )

    def text2image(
        self,
        prompt: str,
        image: Union[Image.Image, str],
        prompt2: Optional[str] = None,
        refiner_prompt: Optional[str] = None,
        refiner_prompt2: Optional[str] = None,
        refiner_negative_prompt: Optional[str] = "",
        refiner_negative_prompt2: Optional[str] = None,
        max_seq_length: Optional[int] = None,
    ):
        if isinstance(image, str):
            image = Image.open(image)
        image = image.convert("RGB")

        original_size = image.size
        image = self.vision_resize(image)
        if self.center_crop:
            y1 = max(0, int(round((image.height - self.image_size) / 2.0)))
            x1 = max(0, int(round((image.width - self.image_size) / 2.0)))
            image = self.vision_crop(image)
        else:
            y1, x1, h, w = self.vision_crop.get_params(
                image, (self.image_size, self.image_size)
            )
            image = crop(image, y1, x1, h, w)
        if self.vision_flip:
            x1 = image.width - x1
            image = self.vision_flip(image)
        crop_top_left = (y1, x1)
        pixel_values = self.vision_processor(image)

        add_time_ids = (
            original_size + crop_top_left + [self.image_size, self.image_size]
        )

        prompt2 = prompt2 or prompt
        prompt_outputs = self.text_processor1.classification(
            prompt, max_seq_length=max_seq_length
        )
        prompt2_outputs = self.text_processor2.classification(
            prompt2, max_seq_length=max_seq_length
        )

        return GenericOutputs(
            pixel_values=pixel_values,
            input_ids=prompt_outputs.input_ids,
            attention_mask=prompt_outputs.attention_mask,
            input2_ids=prompt2_outputs.input_ids,
            attention2_mask=prompt2_outputs.attention_mask,
            add_time_ids=torch.tensor(add_time_ids),
        )

    def text2image_inputs(
        self,
        prompt: str,
        prompt2: Optional[str] = None,
        negative_prompt: Optional[str] = "",
        negative_prompt2: Optional[str] = None,
        refiner_prompt: Optional[str] = None,
        refiner_prompt2: Optional[str] = None,
        refiner_negative_prompt: Optional[str] = "",
        refiner_negative_prompt2: Optional[str] = None,
        max_seq_length: Optional[int] = None,
    ):
        prompt2 = prompt2 or prompt
        negative_prompt2 = negative_prompt2 or negative_prompt
        prompt_outputs = self.text_processor1.classification(
            prompt, max_seq_length=max_seq_length
        )
        prompt2_outputs = self.text_processor2.classification(
            prompt2, max_seq_length=max_seq_length
        )
        negative_prompt_outputs = self.text_processor1.classification(
            negative_prompt, max_seq_length=max_seq_length
        )
        negative_prompt2_outputs = self.text_processor2.classification(
            negative_prompt2, max_seq_length=max_seq_length
        )
        if self.refiner_text_processor1 is not None:
            refiner_prompt = refiner_prompt or prompt
            refiner_negative_prompt = refiner_negative_prompt or negative_prompt

            refiner_prompt_outputs = self.refiner_text_processor1.classification(
                refiner_prompt, max_seq_length=max_seq_length
            )
            refiner_negative_prompt_outputs = (
                self.refiner_text_processor1.classification(
                    refiner_negative_prompt, max_seq_length=max_seq_length
                )
            )
        else:
            refiner_prompt_outputs = None
            refiner_negative_prompt_outputs = None

        if self.refiner_text_processor2 is not None:
            refiner_prompt2 = refiner_prompt2 or refiner_prompt or prompt2
            refiner_negative_prompt2 = (
                refiner_negative_prompt2 or refiner_negative_prompt or negative_prompt2
            )

            refiner_prompt2_outputs = self.refiner_text_processor2.classification(
                refiner_prompt2, max_seq_length=max_seq_length
            )
            refiner_negative_prompt2_outputs = (
                self.refiner_text_processor2.classification(
                    refiner_negative_prompt2, max_seq_length=max_seq_length
                )
            )
        else:
            refiner_prompt2_outputs = None
            refiner_negative_prompt2_outputs = None

        return GenericOutputs(
            input_ids=prompt_outputs.input_ids,
            attention_mask=prompt_outputs.attention_mask,
            input2_ids=prompt2_outputs.input_ids,
            attention2_mask=prompt2_outputs.attention_mask,
            negative_input_ids=negative_prompt_outputs.input_ids,
            negative_attention_mask=negative_prompt_outputs.attention_mask,
            negative_input2_ids=negative_prompt2_outputs.input_ids,
            negative_attention2_mask=negative_prompt2_outputs.attention_mask,
            refiner_input_ids=refiner_prompt_outputs.input_ids
            if refiner_prompt_outputs
            else None,
            refiner_attention_mask=refiner_prompt_outputs.attention_mask
            if refiner_prompt_outputs
            else None,
            refiner_input2_ids=refiner_prompt2_outputs.input_ids
            if refiner_prompt2_outputs
            else None,
            refiner_attention2_mask=refiner_prompt2_outputs.attention_mask
            if refiner_prompt2_outputs
            else None,
            refiner_negative_input_ids=refiner_negative_prompt_outputs.input_ids
            if refiner_negative_prompt_outputs
            else None,
            refiner_negative_attention_mask=refiner_negative_prompt_outputs.attention_mask
            if refiner_negative_prompt_outputs
            else None,
            refiner_negative_input2_ids=refiner_negative_prompt2_outputs.input_ids
            if refiner_negative_prompt2_outputs
            else None,
            refiner_negative_attention2_mask=refiner_negative_prompt2_outputs.attention_mask
            if refiner_negative_prompt2_outputs
            else None,
        )

    def image2image_inputs(
        self,
        prompt: str,
        image: Union[Image.Image, str],
        prompt2: Optional[str] = None,
        negative_prompt: Optional[str] = "",
        negative_prompt2: Optional[str] = None,
        refiner_prompt: Optional[str] = None,
        refiner_prompt2: Optional[str] = None,
        refiner_negative_prompt: Optional[str] = "",
        refiner_negative_prompt2: Optional[str] = None,
        max_seq_length: Optional[int] = None,
    ):
        if isinstance(image, str):
            image = Image.open(image)
        image = image.convert("RGB")
        pixel_values = self.vae_image_processor.preprocess(image)[0]

        text_inputs = self.text2image_inputs(
            prompt,
            prompt2=prompt2,
            negative_prompt=negative_prompt,
            negative_prompt2=negative_prompt2,
            refiner_prompt=refiner_prompt,
            refiner_prompt2=refiner_prompt2,
            refiner_negative_prompt=refiner_negative_prompt,
            refiner_negative_prompt2=refiner_negative_prompt2,
            max_seq_length=max_seq_length,
        )

        return GenericOutputs(
            pixel_values=pixel_values,
            **text_inputs,
        )

    def inpainting_inputs(
        self,
        prompt: str,
        image: Union[Image.Image, str],
        mask_image: Union[Image.Image, str],
        prompt2: Optional[str] = None,
        negative_prompt: Optional[str] = "",
        negative_prompt2: Optional[str] = None,
        refiner_prompt: Optional[str] = None,
        refiner_prompt2: Optional[str] = None,
        refiner_negative_prompt: Optional[str] = "",
        refiner_negative_prompt2: Optional[str] = None,
        max_seq_length: Optional[int] = None,
    ):
        if isinstance(image, str):
            image = Image.open(image)
        image = image.convert("RGB")

        if isinstance(mask_image, str):
            mask_image = Image.open(mask_image)
        mask_image = mask_image.convert("L")

        pixel_values = self.vae_image_processor.preprocess(image)[0]
        pixel_masks = self.vae_image_processor.preprocess(mask_image)[0]
        pixel_masks = (pixel_masks + 1) / 2

        text_inputs = self.text2image_inputs(
            prompt,
            prompt2=prompt2,
            negative_prompt=negative_prompt,
            negative_prompt2=negative_prompt2,
            refiner_prompt=refiner_prompt,
            refiner_prompt2=refiner_prompt2,
            refiner_negative_prompt=refiner_negative_prompt,
            refiner_negative_prompt2=refiner_negative_prompt2,
            max_seq_length=max_seq_length,
        )

        return GenericOutputs(
            pixel_values=pixel_values,
            pixel_masks=pixel_masks,
            **text_inputs,
        )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import logging
import torch
import torch.nn as nn
import torch.nn.functional as F
import transformers
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from transformers import BartConfig, BartModel, BartForConditionalGeneration
from unitorch.utils.decorators import replace
from unitorch.models import GenericModel, GenericOutputs


class BartForGeneration(GenericModel):
    """
    BART model for text generation.
    """

    prefix_keys_in_state_dict = {"^(?!model\.model\.|model\.).*": "model.model."}

    def __init__(
        self,
        config_path: str,
        gradient_checkpointing: Optional[bool] = False,
    ):
        """
        Initializes a BartForGeneration model.

        Args:
            config_path (str): Path to the BART model configuration file.
            gradient_checkpointing (bool, optional): Whether to use gradient checkpointing during training. Defaults to False.
        """
        super().__init__()
        self.config = BartConfig.from_json_file(config_path)
        self.config.gradient_checkpointing = gradient_checkpointing
        self.config.forced_bos_token_id = None
        self.model = BartForConditionalGeneration(self.config)
        self.init_weights()

    def forward(
        self,
        input_ids: torch.Tensor,
        attention_mask: torch.Tensor,
        decoder_input_ids: torch.Tensor,
        decoder_attention_mask: torch.Tensor,
    ):
        """
        Performs forward pass of the BartForGeneration model.

        Args:
            input_ids (torch.Tensor): Tensor of input token IDs.
            attention_mask (torch.Tensor): Tensor indicating which tokens should be attended to.
            decoder_input_ids (torch.Tensor): Tensor of decoder input token IDs.
            decoder_attention_mask (torch.Tensor): Tensor indicating which decoder tokens should be attended to.

        Returns:
            (torch.Tensor): Output logits of the model.
        """
        outputs = self.model(
            input_ids=input_ids,
            attention_mask=attention_mask,
            decoder_input_ids=decoder_input_ids,
            decoder_attention_mask=decoder_attention_mask,
            return_dict=True,
        )
        logits = outputs.logits
        return logits

    @torch.no_grad()
    def generate(
        self,
        input_ids: torch.Tensor,
        num_beams: Optional[int] = 5,
        decoder_start_token_id: Optional[int] = 2,
        decoder_end_token_id: Optional[Union[int, List[int]]] = 2,
        num_return_sequences: Optional[int] = 1,
        min_gen_seq_length: Optional[int] = 0,
        max_gen_seq_length: Optional[int] = 48,
        repetition_penalty: Optional[float] = 1.0,
        no_repeat_ngram_size: Optional[int] = 0,
        early_stopping: Optional[bool] = True,
        length_penalty: Optional[float] = 1.0,
        num_beam_groups: Optional[int] = 1,
        diversity_penalty: Optional[float] = 0.0,
        do_sample: Optional[bool] = False,
        temperature: Optional[float] = 1.0,
        top_k: Optional[int] = 50,
        top_p: Optional[float] = 1.0,
    ):
        """
        Generates text using the BartForGeneration model.

        Args:
            input_ids (torch.Tensor): Tensor of input token IDs.
            num_beams (int, optional): Number of beams for beam search. Defaults to 5.
            decoder_start_token_id (int, optional): The ID of the decoder start token. Defaults to 2.
            decoder_end_token_id (int or List[int], optional): The ID(s) of the decoder end token(s). Defaults to 2.
            num_return_sequences (int, optional): Number of sequences to return. Defaults to 1.
            min_gen_seq_length (int, optional): Minimum length of generated sequences. Defaults to 0.
            max_gen_seq_length (int, optional): Maximum length of generated sequences. Defaults to 48.
            repetition_penalty (float, optional): Repetition penalty for generated sequences. Defaults to 1.0.
            no_repeat_ngram_size (int, optional): Size of n-grams to avoid repetition. Defaults to 0.
            early_stopping (bool, optional): Whether to stop generation early. Defaults to True.
            length_penalty (float, optional): Length penalty for generated sequences. Defaults to 1.0.
            num_beam_groups (int, optional): Number of beam groups for diverse beam search. Defaults to 1.
            diversity_penalty (float, optional): Diversity penalty for diverse beam search. Defaults to 0.0.
            do_sample (bool, optional): Whether to use sampling during generation. Defaults to False.
            temperature (float, optional): Temperature for sampling. Defaults to 1.0.
            top_k (int, optional): Value for top-k sampling. Defaults to 50.
            top_p (float, optional): Value for top-p sampling. Defaults to 1.0.

        Returns:
            GenericOutputs: Generated sequences and their scores.
        """
        outputs = self.model.generate(
            input_ids,
            max_length=max_gen_seq_length,
            min_length=min_gen_seq_length,
            num_beams=num_beams,
            do_sample=do_sample,
            decoder_start_token_id=decoder_start_token_id,
            no_repeat_ngram_size=no_repeat_ngram_size,
            early_stopping=early_stopping,
            length_penalty=length_penalty,
            repetition_penalty=repetition_penalty,
            num_return_sequences=num_return_sequences,
            bos_token_id=decoder_start_token_id,
            eos_token_id=decoder_end_token_id,
            num_beam_groups=num_beam_groups,
            diversity_penalty=diversity_penalty,
            temperature=temperature,
            top_k=top_k,
            top_p=top_p,
            return_dict_in_generate=True,
            output_scores=True,
        )

        sequences = outputs.sequences.reshape(
            -1, num_return_sequences, outputs.sequences.size(-1)
        )
        outputs.sequences = torch.zeros(
            sequences.size(0), num_return_sequences, max_gen_seq_length
        ).to(device=sequences.device)
        outputs.sequences[:, :, : sequences.size(-1)].copy_(sequences)

        if num_return_sequences == 1:
            outputs.sequences = outputs.sequences.reshape(-1, max_gen_seq_length)

        return GenericOutputs(
            sequences=outputs.sequences, sequences_scores=outputs.sequences_scores
        )
